# Copyright (C) 2017-2026 Arun Sivaraman <arunsivaramanneo@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

import gi
import const
import Filenames
import subprocess

gi.require_version("Gtk", "4.0")
gi.require_version(namespace='Adw', version='1')
from gi.repository import Gtk,GObject,Gio,Adw
from Common import  create_scrollbar, createSubTab,getScreenSize,setup,create_tab

FrameBufferToolTip = ["Visual ID", "Visual Depth", "Visual Type", "Transparency", "Buffer Size", "level", "Render Type",
                      "Double Buffer", "Stereo", "Red Colorbuffer Size", "Green Colorbuffer Size",
                      "Blue Colorbuffer Size", "Alpha Colorbuffer Size"
                                               "float", "SRGB", "Auxillary Buffer", "Depth", "Stencil",
                      "Accumbuffer Red", "Accumbuffer Green", "Accumbuffer Blue", "Accumbuffer Alpha", "msnum",
                      "msbufs", "Swap", "Caveats"]

class DataObject(GObject.GObject):
    def __init__(self, data1: str,data2: str,data3: str,data4: str,data5: str, data6: str, data7: str, data8: str, data9: str, data10: str,data11: str,data12: str, data13: str,data14: str,data15: str,data16: str,data17: str, data18: str,data19: str,data20: str,data21: str,data22: str,data23: str,data24: str,data25: str,data26: str):
        super(DataObject, self).__init__()
        self.data1 = data1
        self.data2 = data2
        self.data3 = data3
        self.data4 = data4
        self.data5 = data5
        self.data6 = data6
        self.data7 = data7
        self.data8 = data8
        self.data9 = data9
        self.data10 = data10
        self.data11 = data11
        self.data12 = data12
        self.data13 = data13
        self.data14 = data14
        self.data15 = data15
        self.data16 = data16
        self.data17 = data17
        self.data18 = data18
        self.data19 = data19
        self.data20 = data20
        self.data21 = data21
        self.data22 = data22
        self.data23 = data23
        self.data24 = data24
        self.data25 = data25
        self.data26 = data26

def bind1(widget, item):
    """bind data from the store object to the widget"""
    label = item.get_child()
    obj = item.get_item()
    label.set_label(obj.data1)

def bind2(widget, item):
    """bind data from the store object to the widget"""
    label = item.get_child()
    obj = item.get_item()
    label.set_label(obj.data2)

def bind3(widget, item):
    """bind data from the store object to the widget"""
    label = item.get_child()
    obj = item.get_item()
    label.set_label(obj.data3)

def bind4(widget, item):
    """bind data from the store object to the widget"""
    label = item.get_child()
    obj = item.get_item()
    label.set_label(obj.data4)

def bind5(widget, item):
    """bind data from the store object to the widget"""
    label = item.get_child()
    obj = item.get_item()
    label.set_label(obj.data5)

def bind6(widget, item):
    """bind data from the store object to the widget"""
    label = item.get_child()
    obj = item.get_item()
    label.set_label(obj.data6)

def bind7(widget, item):
    """bind data from the store object to the widget"""
    label = item.get_child()
    obj = item.get_item()
    label.set_label(obj.data7)

def bind8(widget, item):
    """bind data from the store object to the widget"""
    label = item.get_child()
    obj = item.get_item()
    label.set_label(obj.data8)

def bind9(widget, item):
    """bind data from the store object to the widget"""
    label = item.get_child()
    obj = item.get_item()
    label.set_label(obj.data9)

def bind10(widget, item):
    """bind data from the store object to the widget"""
    label = item.get_child()
    obj = item.get_item()
    label.set_label(obj.data10)

def bind11(widget, item):
    """bind data from the store object to the widget"""
    label = item.get_child()
    obj = item.get_item()
    label.set_label(obj.data11)

def bind12(widget, item):
    """bind data from the store object to the widget"""
    label = item.get_child()
    obj = item.get_item()
    label.set_label(obj.data12)

def bind13(widget, item):
    """bind data from the store object to the widget"""
    label = item.get_child()
    obj = item.get_item()
    label.set_label(obj.data13)

def bind14(widget, item):
    """bind data from the store object to the widget"""
    label = item.get_child()
    obj = item.get_item()
    label.set_label(obj.data14)

def bind15(widget, item):
    """bind data from the store object to the widget"""
    label = item.get_child()
    obj = item.get_item()
    label.set_label(obj.data15)

def bind16(widget, item):
    """bind data from the store object to the widget"""
    label = item.get_child()
    obj = item.get_item()
    label.set_label(obj.data16)

def bind17(widget, item):
    """bind data from the store object to the widget"""
    label = item.get_child()
    obj = item.get_item()
    label.set_label(obj.data17)

def bind18(widget, item):
    """bind data from the store object to the widget"""
    label = item.get_child()
    obj = item.get_item()
    label.set_label(obj.data18)

def bind19(widget, item):
    """bind data from the store object to the widget"""
    label = item.get_child()
    obj = item.get_item()
    label.set_label(obj.data19)

def bind20(widget, item):
    """bind data from the store object to the widget"""
    label = item.get_child()
    obj = item.get_item()
    label.set_label(obj.data20)

def bind21(widget, item):
    """bind data from the store object to the widget"""
    label = item.get_child()
    obj = item.get_item()
    label.set_label(obj.data22)

def bind22(widget, item):
    """bind data from the store object to the widget"""
    label = item.get_child()
    obj = item.get_item()
    label.set_label(obj.data22)

def bind23(widget, item):
    """bind data from the store object to the widget"""
    label = item.get_child()
    obj = item.get_item()
    label.set_label(obj.data23)

def bind24(widget, item):
    """bind data from the store object to the widget"""
    label = item.get_child()
    obj = item.get_item()
    label.set_label(obj.data24)

def bind25(widget, item):
    """bind data from the store object to the widget"""
    label = item.get_child()
    obj = item.get_item()
    label.set_label(obj.data25)

def bind26(widget, item):
    """bind data from the store object to the widget"""
    label = item.get_child()
    obj = item.get_item()
    label.set_label(obj.data26)

def FrameBuffer(button):
    FBWin = Adw.Window()
    FBWin.set_title("GLX Frame Buffer Configuration")
    headerbar = Adw.HeaderBar.new()
    headerbar.add_css_class(css_class='compact')

    adw_toolbar_view = Adw.ToolbarView.new()
    FBWin.set_content(adw_toolbar_view)

    adw_toolbar_view.add_top_bar(headerbar)
    #   FBWin.set_size_request(1000, 500)
 #   setScreenSize(FBWin, const.WIDTH_RATIO, const.HEIGHT_RATIO2)

    FBNotebook = Adw.ViewStack.new()
    FBNotebook_stack_switcher = Adw.ViewSwitcher.new()
    FBNotebook_stack_switcher.set_stack(stack=FBNotebook)
    FBNotebook_stack_switcher.set_policy(1)
    adw_toolbar_view.set_content(FBNotebook)
    headerbar.set_title_widget(title_widget=FBNotebook_stack_switcher)

#    FBGLX_Store = Gtk.ListStore(str, str, str, str, str, str, str, str, str, str, str, str, str, str, str, str, str,
#                                str,
#                                str, str, str, str, str, str, str, str, str)
#    TreeFBGLX = Gtk.TreeView.new_with_model(FBGLX_Store)
#    TreeFBGLX.set_enable_search(True)
#    TreeFBGLX.set_property("enable-grid-lines", 3)


    button.set_sensitive(False)

    fetch_framebuffer_glx_visual_command = "cat %s  | awk '/GLX Visuals.*/{flag=1;next}/GLXFBConfigs.*/{flag=0}flag' | awk '/----.*/{flag=1;next}flag' " %(Filenames.opengl_outpuf_file)

    fetch_framebuffer_glx_visual_process = subprocess.Popen(fetch_framebuffer_glx_visual_command,shell=True,stdout=subprocess.PIPE,universal_newlines=True)
    list_glx_visuals= fetch_framebuffer_glx_visual_process.communicate()[0].splitlines()

    list_glx_visuals = [i.split() for i in list_glx_visuals]

    fetch_framebuffer_glx_fbconfigs_command = "cat %s | awk '/GLXFBConfigs.*/{flag=1;next}flag' | awk '/----.*/{flag=1;next}flag' " %(Filenames.opengl_outpuf_file)

    fetch_framebuffer_glx_fbconfigs_process= subprocess.Popen(fetch_framebuffer_glx_fbconfigs_command,shell=True,stdout=subprocess.PIPE,universal_newlines=True)
    list_fb_configs = fetch_framebuffer_glx_fbconfigs_process.communicate()[0].splitlines()
    
    list_fb_configs = [i.split() for i in list_fb_configs]    

    frameBufferColumnView = Gtk.ColumnView()
    frameBufferColumnView.props.show_row_separators = True
    frameBufferColumnView.props.show_column_separators = True

    factory_fbglx_value1 = Gtk.SignalListItemFactory()
    factory_fbglx_value1.connect("setup",setup)
    factory_fbglx_value1.connect("bind",bind1)

    fbglxColumn1 = Gtk.ColumnViewColumn.new(const.FRAMEBUFFERLIST[0],factory_fbglx_value1)
    fbglxColumn1.set_resizable(True)
    frameBufferColumnView.append_column(fbglxColumn1)

    factory_fbglx_value2 = Gtk.SignalListItemFactory()
    factory_fbglx_value2.connect("setup",setup)
    factory_fbglx_value2.connect("bind",bind2)

    fbglxColumn2 = Gtk.ColumnViewColumn.new(const.FRAMEBUFFERLIST[1],factory_fbglx_value2)
    fbglxColumn2.set_resizable(True)
    frameBufferColumnView.append_column(fbglxColumn2)

    factory_fbglx_value3 = Gtk.SignalListItemFactory()
    factory_fbglx_value3.connect("setup",setup)
    factory_fbglx_value3.connect("bind",bind3)

    fbglxColumn3 = Gtk.ColumnViewColumn.new(const.FRAMEBUFFERLIST[2],factory_fbglx_value3)
    fbglxColumn3.set_resizable(True)
    frameBufferColumnView.append_column(fbglxColumn3)

    factory_fbglx_value4 = Gtk.SignalListItemFactory()
    factory_fbglx_value4.connect("setup",setup)
    factory_fbglx_value4.connect("bind",bind4)

    fbglxColumn4 = Gtk.ColumnViewColumn.new(const.FRAMEBUFFERLIST[3],factory_fbglx_value4)
    fbglxColumn4.set_resizable(True)
    frameBufferColumnView.append_column(fbglxColumn4)

    factory_fbglx_value5 = Gtk.SignalListItemFactory()
    factory_fbglx_value5.connect("setup",setup)
    factory_fbglx_value5.connect("bind",bind5)

    fbglxColumn5 = Gtk.ColumnViewColumn.new(const.FRAMEBUFFERLIST[4],factory_fbglx_value5)
    fbglxColumn5.set_resizable(True)
    frameBufferColumnView.append_column(fbglxColumn5)

    factory_fbglx_value6 = Gtk.SignalListItemFactory()
    factory_fbglx_value6.connect("setup",setup)
    factory_fbglx_value6.connect("bind",bind6)

    fbglxColumn6 = Gtk.ColumnViewColumn.new(const.FRAMEBUFFERLIST[5],factory_fbglx_value6)
    fbglxColumn6.set_resizable(True)
    frameBufferColumnView.append_column(fbglxColumn6)

    factory_fbglx_value7 = Gtk.SignalListItemFactory()
    factory_fbglx_value7.connect("setup",setup)
    factory_fbglx_value7.connect("bind",bind7)

    fbglxColumn7 = Gtk.ColumnViewColumn.new(const.FRAMEBUFFERLIST[6],factory_fbglx_value7)
    fbglxColumn7.set_resizable(True)
    frameBufferColumnView.append_column(fbglxColumn7)

    factory_fbglx_value8 = Gtk.SignalListItemFactory()
    factory_fbglx_value8.connect("setup",setup)
    factory_fbglx_value8.connect("bind",bind8)

    fbglxColumn8 = Gtk.ColumnViewColumn.new(const.FRAMEBUFFERLIST[7],factory_fbglx_value8)
    fbglxColumn8.set_resizable(True)
    frameBufferColumnView.append_column(fbglxColumn8)

    factory_fbglx_value9 = Gtk.SignalListItemFactory()
    factory_fbglx_value9.connect("setup",setup)
    factory_fbglx_value9.connect("bind",bind9)

    fbglxColumn9 = Gtk.ColumnViewColumn.new(const.FRAMEBUFFERLIST[8],factory_fbglx_value9)
    fbglxColumn9.set_resizable(True)
    frameBufferColumnView.append_column(fbglxColumn9)

    factory_fbglx_value10 = Gtk.SignalListItemFactory()
    factory_fbglx_value10.connect("setup",setup)
    factory_fbglx_value10.connect("bind",bind10)

    fbglxColumn10 = Gtk.ColumnViewColumn.new(const.FRAMEBUFFERLIST[9],factory_fbglx_value10)
    fbglxColumn10.set_resizable(True)
    frameBufferColumnView.append_column(fbglxColumn10)

    factory_fbglx_value11 = Gtk.SignalListItemFactory()
    factory_fbglx_value11.connect("setup",setup)
    factory_fbglx_value11.connect("bind",bind11)

    fbglxColumn11 = Gtk.ColumnViewColumn.new(const.FRAMEBUFFERLIST[10],factory_fbglx_value11)
    fbglxColumn11.set_resizable(True)
    frameBufferColumnView.append_column(fbglxColumn11)

    factory_fbglx_value12 = Gtk.SignalListItemFactory()
    factory_fbglx_value12.connect("setup",setup)
    factory_fbglx_value12.connect("bind",bind12)

    fbglxColumn12 = Gtk.ColumnViewColumn.new(const.FRAMEBUFFERLIST[11],factory_fbglx_value12)
    fbglxColumn12.set_resizable(True)
    frameBufferColumnView.append_column(fbglxColumn12)

    factory_fbglx_value13 = Gtk.SignalListItemFactory()
    factory_fbglx_value13.connect("setup",setup)
    factory_fbglx_value13.connect("bind",bind13)

    fbglxColumn13 = Gtk.ColumnViewColumn.new(const.FRAMEBUFFERLIST[12],factory_fbglx_value13)
    fbglxColumn13.set_resizable(True)
    frameBufferColumnView.append_column(fbglxColumn13)

    factory_fbglx_value14 = Gtk.SignalListItemFactory()
    factory_fbglx_value14.connect("setup",setup)
    factory_fbglx_value14.connect("bind",bind14)

    fbglxColumn14 = Gtk.ColumnViewColumn.new(const.FRAMEBUFFERLIST[13],factory_fbglx_value14)
    fbglxColumn14.set_resizable(True)
    frameBufferColumnView.append_column(fbglxColumn14)

    factory_fbglx_value15 = Gtk.SignalListItemFactory()
    factory_fbglx_value15.connect("setup",setup)
    factory_fbglx_value15.connect("bind",bind15)

    fbglxColumn15 = Gtk.ColumnViewColumn.new(const.FRAMEBUFFERLIST[14],factory_fbglx_value15)
    fbglxColumn15.set_resizable(True)
    frameBufferColumnView.append_column(fbglxColumn15)

    factory_fbglx_value16 = Gtk.SignalListItemFactory()
    factory_fbglx_value16.connect("setup",setup)
    factory_fbglx_value16.connect("bind",bind16)

    fbglxColumn16 = Gtk.ColumnViewColumn.new(const.FRAMEBUFFERLIST[15],factory_fbglx_value16)
    fbglxColumn16.set_resizable(True)
    frameBufferColumnView.append_column(fbglxColumn16)

    factory_fbglx_value17 = Gtk.SignalListItemFactory()
    factory_fbglx_value17.connect("setup",setup)
    factory_fbglx_value17.connect("bind",bind17)

    fbglxColumn17 = Gtk.ColumnViewColumn.new(const.FRAMEBUFFERLIST[16],factory_fbglx_value17)
    fbglxColumn17.set_resizable(True)
    frameBufferColumnView.append_column(fbglxColumn17)

    factory_fbglx_value18 = Gtk.SignalListItemFactory()
    factory_fbglx_value18.connect("setup",setup)
    factory_fbglx_value18.connect("bind",bind18)

    fbglxColumn18 = Gtk.ColumnViewColumn.new(const.FRAMEBUFFERLIST[17],factory_fbglx_value18)
    fbglxColumn18.set_resizable(True)
    frameBufferColumnView.append_column(fbglxColumn18)

    factory_fbglx_value19 = Gtk.SignalListItemFactory()
    factory_fbglx_value19.connect("setup",setup)
    factory_fbglx_value19.connect("bind",bind19)

    fbglxColumn19 = Gtk.ColumnViewColumn.new(const.FRAMEBUFFERLIST[18],factory_fbglx_value19)
    fbglxColumn19.set_resizable(True)
    frameBufferColumnView.append_column(fbglxColumn19)

    factory_fbglx_value20 = Gtk.SignalListItemFactory()
    factory_fbglx_value20.connect("setup",setup)
    factory_fbglx_value20.connect("bind",bind20)

    fbglxColumn20 = Gtk.ColumnViewColumn.new(const.FRAMEBUFFERLIST[19],factory_fbglx_value20)
    fbglxColumn20.set_resizable(True)
    frameBufferColumnView.append_column(fbglxColumn20)

    factory_fbglx_value21 = Gtk.SignalListItemFactory()
    factory_fbglx_value21.connect("setup",setup)
    factory_fbglx_value21.connect("bind",bind21)

    fbglxColumn21 = Gtk.ColumnViewColumn.new(const.FRAMEBUFFERLIST[20],factory_fbglx_value21)
    fbglxColumn21.set_resizable(True)
    frameBufferColumnView.append_column(fbglxColumn21)

    factory_fbglx_value22 = Gtk.SignalListItemFactory()
    factory_fbglx_value22.connect("setup",setup)
    factory_fbglx_value22.connect("bind",bind22)

    fbglxColumn22 = Gtk.ColumnViewColumn.new(const.FRAMEBUFFERLIST[21],factory_fbglx_value22)
    fbglxColumn22.set_resizable(True)
    frameBufferColumnView.append_column(fbglxColumn22)

    factory_fbglx_value23 = Gtk.SignalListItemFactory()
    factory_fbglx_value23.connect("setup",setup)
    factory_fbglx_value23.connect("bind",bind23)

    fbglxColumn23 = Gtk.ColumnViewColumn.new(const.FRAMEBUFFERLIST[22],factory_fbglx_value23)
    fbglxColumn23.set_resizable(True)
    frameBufferColumnView.append_column(fbglxColumn23)

    factory_fbglx_value24 = Gtk.SignalListItemFactory()
    factory_fbglx_value24.connect("setup",setup)
    factory_fbglx_value24.connect("bind",bind24)

    fbglxColumn24 = Gtk.ColumnViewColumn.new(const.FRAMEBUFFERLIST[23],factory_fbglx_value24)
    fbglxColumn24.set_resizable(True)
    frameBufferColumnView.append_column(fbglxColumn24)

    factory_fbglx_value25 = Gtk.SignalListItemFactory()
    factory_fbglx_value25.connect("setup",setup)
    factory_fbglx_value25.connect("bind",bind25)

    fbglxColumn25 = Gtk.ColumnViewColumn.new(const.FRAMEBUFFERLIST[24],factory_fbglx_value25)
    fbglxColumn25.set_resizable(True)
    frameBufferColumnView.append_column(fbglxColumn25)

    factory_fbglx_value26 = Gtk.SignalListItemFactory()
    factory_fbglx_value26.connect("setup",setup)
    factory_fbglx_value26.connect("bind",bind26)

    fbglxColumn26 = Gtk.ColumnViewColumn.new(const.FRAMEBUFFERLIST[25],factory_fbglx_value26)
    fbglxColumn26.set_resizable(True)
    fbglxColumn26.set_expand(True)
    frameBufferColumnView.append_column(fbglxColumn26)

    frameBufferSelection = Gtk.SingleSelection()
    FBGLX_Store = Gio.ListStore.new(DataObject)
    frameBufferSelection.set_model(FBGLX_Store)
    frameBufferColumnView.set_model(frameBufferSelection)
    
    for i in range(len(list_glx_visuals) - 1):
        FBGLX_Store.append(DataObject(list_glx_visuals[i][0],list_glx_visuals[i][1],list_glx_visuals[i][2],list_glx_visuals[i][3],list_glx_visuals[i][4],list_glx_visuals[i][5],list_glx_visuals[i][6],list_glx_visuals[i][7],list_glx_visuals[i][8],list_glx_visuals[i][9],list_glx_visuals[i][10],list_glx_visuals[i][11],list_glx_visuals[i][12],list_glx_visuals[i][13],list_glx_visuals[i][14],list_glx_visuals[i][15],list_glx_visuals[i][16],list_glx_visuals[i][17],list_glx_visuals[i][18],list_glx_visuals[i][19],list_glx_visuals[i][20],list_glx_visuals[i][21],list_glx_visuals[i][22],list_glx_visuals[i][23],list_glx_visuals[i][24],list_glx_visuals[i][25]))
    label1 = "%d GLX Visuals" % (len(list_glx_visuals) - 1)
#    FBNotebook.set_tab_label(FBGLXTab, Gtk.Label(label=label1))
    
    FBGLXTab = create_tab(FBNotebook,"settings",label1,20,True)
    FBGLXGrid = Gtk.Grid()
    FBGLXTab.append(FBGLXGrid)

#    setColumnFrameBuffer(TreeFBGLX, const.FRAMEBUFFERLIST)

    FBGLXScrollbar = create_scrollbar(frameBufferColumnView)
    FBGLXGrid.attach(FBGLXScrollbar,0,0,1,1)


    frameBufferConfigColumnView = Gtk.ColumnView()
    frameBufferConfigColumnView.props.show_row_separators = True
    frameBufferConfigColumnView.props.show_column_separators = True


    factory_fbconfig_value1 = Gtk.SignalListItemFactory()
    factory_fbconfig_value1.connect("setup",setup)
    factory_fbconfig_value1.connect("bind",bind1)

    fbconfigColumn1 = Gtk.ColumnViewColumn.new(const.FRAMEBUFFERLIST[0],factory_fbconfig_value1)
    fbconfigColumn1.set_resizable(True)
    frameBufferConfigColumnView.append_column(fbconfigColumn1)

    factory_fbconfig_value2 = Gtk.SignalListItemFactory()
    factory_fbconfig_value2.connect("setup",setup)
    factory_fbconfig_value2.connect("bind",bind2)

    fbconfigColumn2 = Gtk.ColumnViewColumn.new(const.FRAMEBUFFERLIST[1],factory_fbconfig_value2)
    fbconfigColumn2.set_resizable(True)
    frameBufferConfigColumnView.append_column(fbconfigColumn2)

    factory_fbconfig_value3 = Gtk.SignalListItemFactory()
    factory_fbconfig_value3.connect("setup",setup)
    factory_fbconfig_value3.connect("bind",bind3)

    fbconfigColumn3 = Gtk.ColumnViewColumn.new(const.FRAMEBUFFERLIST[2],factory_fbconfig_value3)
    fbconfigColumn3.set_resizable(True)
    frameBufferConfigColumnView.append_column(fbconfigColumn3)

    factory_fbconfig_value4 = Gtk.SignalListItemFactory()
    factory_fbconfig_value4.connect("setup",setup)
    factory_fbconfig_value4.connect("bind",bind4)

    fbconfigColumn4 = Gtk.ColumnViewColumn.new(const.FRAMEBUFFERLIST[3],factory_fbconfig_value4)
    fbconfigColumn4.set_resizable(True)
    frameBufferConfigColumnView.append_column(fbconfigColumn4)

    factory_fbconfig_value5 = Gtk.SignalListItemFactory()
    factory_fbconfig_value5.connect("setup",setup)
    factory_fbconfig_value5.connect("bind",bind5)

    fbconfigColumn5 = Gtk.ColumnViewColumn.new(const.FRAMEBUFFERLIST[4],factory_fbconfig_value5)
    fbconfigColumn5.set_resizable(True)
    frameBufferConfigColumnView.append_column(fbconfigColumn5)

    factory_fbconfig_value6 = Gtk.SignalListItemFactory()
    factory_fbconfig_value6.connect("setup",setup)
    factory_fbconfig_value6.connect("bind",bind6)

    fbconfigColumn6 = Gtk.ColumnViewColumn.new(const.FRAMEBUFFERLIST[5],factory_fbconfig_value6)
    fbconfigColumn6.set_resizable(True)
    frameBufferConfigColumnView.append_column(fbconfigColumn6)

    factory_fbconfig_value7 = Gtk.SignalListItemFactory()
    factory_fbconfig_value7.connect("setup",setup)
    factory_fbconfig_value7.connect("bind",bind7)

    fbconfigColumn7 = Gtk.ColumnViewColumn.new(const.FRAMEBUFFERLIST[6],factory_fbconfig_value7)
    fbconfigColumn7.set_resizable(True)
    frameBufferConfigColumnView.append_column(fbconfigColumn7)

    factory_fbconfig_value8 = Gtk.SignalListItemFactory()
    factory_fbconfig_value8.connect("setup",setup)
    factory_fbconfig_value8.connect("bind",bind8)

    fbconfigColumn8 = Gtk.ColumnViewColumn.new(const.FRAMEBUFFERLIST[7],factory_fbconfig_value8)
    fbconfigColumn8.set_resizable(True)
    frameBufferConfigColumnView.append_column(fbconfigColumn8)

    factory_fbconfig_value9 = Gtk.SignalListItemFactory()
    factory_fbconfig_value9.connect("setup",setup)
    factory_fbconfig_value9.connect("bind",bind9)

    fbconfigColumn9 = Gtk.ColumnViewColumn.new(const.FRAMEBUFFERLIST[8],factory_fbconfig_value9)
    fbconfigColumn9.set_resizable(True)
    frameBufferConfigColumnView.append_column(fbconfigColumn9)

    factory_fbconfig_value10 = Gtk.SignalListItemFactory()
    factory_fbconfig_value10.connect("setup",setup)
    factory_fbconfig_value10.connect("bind",bind10)

    fbconfigColumn10 = Gtk.ColumnViewColumn.new(const.FRAMEBUFFERLIST[9],factory_fbconfig_value10)
    fbconfigColumn10.set_resizable(True)
    frameBufferConfigColumnView.append_column(fbconfigColumn10)

    factory_fbconfig_value11 = Gtk.SignalListItemFactory()
    factory_fbconfig_value11.connect("setup",setup)
    factory_fbconfig_value11.connect("bind",bind11)

    fbconfigColumn11 = Gtk.ColumnViewColumn.new(const.FRAMEBUFFERLIST[10],factory_fbconfig_value11)
    fbconfigColumn11.set_resizable(True)
    frameBufferConfigColumnView.append_column(fbconfigColumn11)

    factory_fbconfig_value12 = Gtk.SignalListItemFactory()
    factory_fbconfig_value12.connect("setup",setup)
    factory_fbconfig_value12.connect("bind",bind12)

    fbconfigColumn12 = Gtk.ColumnViewColumn.new(const.FRAMEBUFFERLIST[11],factory_fbconfig_value12)
    fbconfigColumn12.set_resizable(True)
    frameBufferConfigColumnView.append_column(fbconfigColumn12)

    factory_fbconfig_value13 = Gtk.SignalListItemFactory()
    factory_fbconfig_value13.connect("setup",setup)
    factory_fbconfig_value13.connect("bind",bind13)

    fbconfigColumn13 = Gtk.ColumnViewColumn.new(const.FRAMEBUFFERLIST[12],factory_fbconfig_value13)
    fbconfigColumn13.set_resizable(True)
    frameBufferConfigColumnView.append_column(fbconfigColumn13)

    factory_fbconfig_value14 = Gtk.SignalListItemFactory()
    factory_fbconfig_value14.connect("setup",setup)
    factory_fbconfig_value14.connect("bind",bind14)

    fbconfigColumn14 = Gtk.ColumnViewColumn.new(const.FRAMEBUFFERLIST[13],factory_fbconfig_value14)
    fbconfigColumn14.set_resizable(True)
    frameBufferConfigColumnView.append_column(fbconfigColumn14)

    factory_fbconfig_value15 = Gtk.SignalListItemFactory()
    factory_fbconfig_value15.connect("setup",setup)
    factory_fbconfig_value15.connect("bind",bind15)

    fbconfigColumn15 = Gtk.ColumnViewColumn.new(const.FRAMEBUFFERLIST[14],factory_fbconfig_value15)
    fbconfigColumn15.set_resizable(True)
    frameBufferConfigColumnView.append_column(fbconfigColumn15)

    factory_fbconfig_value16 = Gtk.SignalListItemFactory()
    factory_fbconfig_value16.connect("setup",setup)
    factory_fbconfig_value16.connect("bind",bind16)

    fbconfigColumn16 = Gtk.ColumnViewColumn.new(const.FRAMEBUFFERLIST[15],factory_fbconfig_value16)
    fbconfigColumn16.set_resizable(True)
    frameBufferConfigColumnView.append_column(fbconfigColumn16)

    factory_fbconfig_value17 = Gtk.SignalListItemFactory()
    factory_fbconfig_value17.connect("setup",setup)
    factory_fbconfig_value17.connect("bind",bind17)

    fbconfigColumn17 = Gtk.ColumnViewColumn.new(const.FRAMEBUFFERLIST[16],factory_fbconfig_value17)
    fbconfigColumn17.set_resizable(True)
    frameBufferConfigColumnView.append_column(fbconfigColumn17)

    factory_fbconfig_value18 = Gtk.SignalListItemFactory()
    factory_fbconfig_value18.connect("setup",setup)
    factory_fbconfig_value18.connect("bind",bind18)

    fbconfigColumn18 = Gtk.ColumnViewColumn.new(const.FRAMEBUFFERLIST[17],factory_fbconfig_value18)
    fbconfigColumn18.set_resizable(True)
    frameBufferConfigColumnView.append_column(fbconfigColumn18)

    factory_fbconfig_value19 = Gtk.SignalListItemFactory()
    factory_fbconfig_value19.connect("setup",setup)
    factory_fbconfig_value19.connect("bind",bind19)

    fbconfigColumn19 = Gtk.ColumnViewColumn.new(const.FRAMEBUFFERLIST[18],factory_fbconfig_value19)
    fbconfigColumn19.set_resizable(True)
    frameBufferConfigColumnView.append_column(fbconfigColumn19)

    factory_fbconfig_value20 = Gtk.SignalListItemFactory()
    factory_fbconfig_value20.connect("setup",setup)
    factory_fbconfig_value20.connect("bind",bind20)

    fbconfigColumn20 = Gtk.ColumnViewColumn.new(const.FRAMEBUFFERLIST[19],factory_fbconfig_value20)
    fbconfigColumn20.set_resizable(True)
    frameBufferConfigColumnView.append_column(fbconfigColumn20)

    factory_fbconfig_value21 = Gtk.SignalListItemFactory()
    factory_fbconfig_value21.connect("setup",setup)
    factory_fbconfig_value21.connect("bind",bind21)

    fbconfigColumn21 = Gtk.ColumnViewColumn.new(const.FRAMEBUFFERLIST[20],factory_fbconfig_value21)
    fbconfigColumn21.set_resizable(True)
    frameBufferConfigColumnView.append_column(fbconfigColumn21)

    factory_fbconfig_value22 = Gtk.SignalListItemFactory()
    factory_fbconfig_value22.connect("setup",setup)
    factory_fbconfig_value22.connect("bind",bind22)

    fbconfigColumn22 = Gtk.ColumnViewColumn.new(const.FRAMEBUFFERLIST[21],factory_fbconfig_value22)
    fbconfigColumn22.set_resizable(True)
    frameBufferConfigColumnView.append_column(fbconfigColumn22)

    factory_fbconfig_value23 = Gtk.SignalListItemFactory()
    factory_fbconfig_value23.connect("setup",setup)
    factory_fbconfig_value23.connect("bind",bind23)

    fbconfigColumn23 = Gtk.ColumnViewColumn.new(const.FRAMEBUFFERLIST[22],factory_fbconfig_value23)
    fbconfigColumn23.set_resizable(True)
    frameBufferConfigColumnView.append_column(fbconfigColumn23)

    factory_fbconfig_value24 = Gtk.SignalListItemFactory()
    factory_fbconfig_value24.connect("setup",setup)
    factory_fbconfig_value24.connect("bind",bind24)

    fbconfigColumn24 = Gtk.ColumnViewColumn.new(const.FRAMEBUFFERLIST[23],factory_fbconfig_value24)
    fbconfigColumn24.set_resizable(True)
    frameBufferConfigColumnView.append_column(fbconfigColumn24)

    factory_fbconfig_value25 = Gtk.SignalListItemFactory()
    factory_fbconfig_value25.connect("setup",setup)
    factory_fbconfig_value25.connect("bind",bind25)

    fbconfigColumn25 = Gtk.ColumnViewColumn.new(const.FRAMEBUFFERLIST[24],factory_fbconfig_value25)
    fbconfigColumn25.set_resizable(True)
    frameBufferConfigColumnView.append_column(fbconfigColumn25)

    factory_fbconfig_value26 = Gtk.SignalListItemFactory()
    factory_fbconfig_value26.connect("setup",setup)
    factory_fbconfig_value26.connect("bind",bind26)

    fbconfigColumn26 = Gtk.ColumnViewColumn.new(const.FRAMEBUFFERLIST[25],factory_fbconfig_value26)
    fbconfigColumn26.set_resizable(True)
    fbconfigColumn26.set_expand(True)
    frameBufferConfigColumnView.append_column(fbconfigColumn26)

    frameBufferConfigSelection = Gtk.SingleSelection()
    FBConfig_Store = Gio.ListStore.new(DataObject)
    frameBufferConfigSelection.set_model(FBConfig_Store)
    frameBufferConfigColumnView.set_model(frameBufferConfigSelection)

    for i in range(len(list_fb_configs) - 1):
        if list_fb_configs[i][6] == "r" or list_fb_configs[i][6] == "c":
            pass
        else:
            list_fb_configs[i].insert(6, ".")
        FBConfig_Store.append(DataObject(list_fb_configs[i][0],list_fb_configs[i][1],list_fb_configs[i][2],list_fb_configs[i][3],list_fb_configs[i][4],list_fb_configs[i][5],list_fb_configs[i][6],list_fb_configs[i][7],list_fb_configs[i][8],list_fb_configs[i][9],list_fb_configs[i][10],list_fb_configs[i][11],list_fb_configs[i][12],list_fb_configs[i][13],list_fb_configs[i][14],list_fb_configs[i][15],list_fb_configs[i][16],list_fb_configs[i][17],list_fb_configs[i][18],list_fb_configs[i][19],list_fb_configs[i][20],list_fb_configs[i][21],list_fb_configs[i][22],list_fb_configs[i][23],list_fb_configs[i][24],list_fb_configs[i][25]))
    label2 = "%d  GLX FBConfigs" % (len(list_fb_configs) - 1)
#    FBNotebook.set_tab_label(FBConfigTab, Gtk.Label(label=label2))
    FBConfigTab = create_tab(FBNotebook,"settings",label2,20,True)
    FBConfigGrid = Gtk.Grid()
    FBConfigTab.append(FBConfigGrid)
#    setColumnFrameBuffer(TreeFBConfig, const.FRAMEBUFFERLIST)

    FBConfigScrollbar = create_scrollbar(frameBufferConfigColumnView)
    FBConfigGrid.attach(FBConfigScrollbar,0,0,1,1)

    def button_enable(win):
        button.set_sensitive(True)

    FBWin.connect("close-request", button_enable)
    screen_width,screen_height = getScreenSize()
    FBWin.set_size_request(960,640)
    FBWin.present()

    # Gtk.main()



#!/usr/bin/env python3

# Copyright (C) 2017-2026 Arun Sivaraman <arunsivaramanneo@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

# Import necessary libraries.
import sys
import gi

# This line is crucial for working with PyGObject.
# It ensures we are using the correct versions of the libraries.
gi.require_version("Gtk", "4.0")
gi.require_version("Adw", "1")
from gi.repository import Gtk, Adw, GObject, Gio

# Define a custom GObject to hold our data. This is the modern,
# idiomatic way to handle list models in GTK4.
class DataItem(GObject.Object):
    def __init__(self, name, category, value):
        super().__init__()
        self.name = name
        self.category = category
        self.value = value

    # Define GObject properties for the data.
    @GObject.Property(type=str)
    def name(self):
        return self._name

    @name.setter
    def name(self, value):
        self._name = value

    @GObject.Property(type=str)
    def category(self):
        return self._category

    @category.setter
    def category(self, value):
        self._category = value

    @GObject.Property(type=int)
    def value(self):
        return self._value

    @value.setter
    def value(self, value):
        self._value = value

# Define the main application class.
# It inherits from Adw.Application, which provides a modern application shell.
class MyApp(Adw.Application):
    def __init__(self, **kwargs):
        # Call the parent class's constructor.
        # Set the application ID, which is a unique identifier.
        super().__init__(application_id="com.example.SidebarApp", **kwargs)
        self.connect("activate", self.on_activate)

    def on_activate(self, app):
        """
        This method is called when the application is activated.
        It's the main entry point for setting up the UI.
        """
        # Changed to Adw.ViewStack for compatibility with Adw.ViewSwitcher.
        self.content_stack = Adw.ViewStack()

        # Gtk.StackSidebar is not compatible with Adw.ViewStack, so it has been removed.
        # self.sidebar = Gtk.StackSidebar()
        # self.sidebar.set_stack(self.content_stack)

        # Create a new Adw.NavigationPage to act as the container for the sidebar.
        # This page will now be empty.
        sidebar_page = Adw.NavigationPage.new(Gtk.Box(), "Navigation")

        # Create a main navigation view.
        self.nav_view = Adw.NavigationSplitView(vexpand=True)
        self.nav_view.set_sidebar(sidebar_page)
        
        # The content also needs to be wrapped in an Adw.NavigationPage.
        content_page = Adw.NavigationPage.new(self.content_stack, "Content")
        self.nav_view.set_content(content_page)

        # Create the widgets for the new dropdown section.
        # FIX: Moved this code block to make the widgets available for the home_box.
        device_label = Gtk.Label(label="Available Device(s):")
        # Use Gtk.StringList for the dropdown's model with sample data.
        device_model = Gtk.StringList.new(["Device 1", "Device 2", "Device 3"])
        device_dropdown = Gtk.DropDown.new(device_model, None)

        # Create a Gtk.Box to hold the device selection widgets.
        device_selection_box = Gtk.Box(
            orientation=Gtk.Orientation.HORIZONTAL,
            spacing=10,
            margin_start=10,
            margin_end=10
        )
        device_selection_box.append(device_label)
        device_selection_box.append(device_dropdown)

        # Add all pages to the Adw.ViewStack.
        home_box = Gtk.Box(
            orientation=Gtk.Orientation.VERTICAL,
            spacing=10,
            vexpand=True
        )
        # FIX: Appended the device selection box to the home_box.
        home_box.append(device_selection_box)
        home_label = Gtk.Label(label="Welcome to the Home Page!", halign=Gtk.Align.CENTER, valign=Gtk.Align.CENTER)
        home_button = Gtk.Button(label="Go to Settings", icon_name="open-menu-symbolic", halign=Gtk.Align.CENTER)
        home_box.append(home_label)
        home_box.append(home_button)
        # Using Adw.ViewStack.add_titled_with_icon to add pages for the view switcher.
        self.content_stack.add_titled_with_icon(home_box, "home", "Home", "go-home-symbolic")

        settings_box = Gtk.Box(
            orientation=Gtk.Orientation.VERTICAL,
            halign=Gtk.Align.CENTER,
            valign=Gtk.Align.CENTER,
            spacing=10,
            vexpand=True
        )
        settings_label = Gtk.Label(label="This is the Settings Page.")
        settings_box.append(settings_label)
        self.content_stack.add_titled_with_icon(settings_box, "settings", "Settings", "preferences-system-symbolic")

        info_box = Gtk.Box(
            orientation=Gtk.Orientation.VERTICAL,
            halign=Gtk.Align.CENTER,
            valign=Gtk.Align.CENTER,
            spacing=10,
            vexpand=True
        )
        info_label = Gtk.Label(label="This is the Info Page.")
        info_box.append(info_label)
        self.content_stack.add_titled_with_icon(info_box, "info", "Info", "dialog-information-symbolic")

        logs_box = Gtk.Box(
            orientation=Gtk.Orientation.VERTICAL,
            halign=Gtk.Align.CENTER,
            valign=Gtk.Align.CENTER,
            spacing=10,
            vexpand=True
        )
        logs_label = Gtk.Label(label="This is the Logs Page.")
        logs_box.append(logs_label)
        self.content_stack.add_titled_with_icon(logs_box, "logs", "Logs", "document-symbolic")

        # Create a new page with a Gtk.ColumnView.
        data_box = Gtk.Box(
            orientation=Gtk.Orientation.VERTICAL,
            vexpand=True,
            hexpand=True,
            spacing=10,
            margin_top=10,
            margin_bottom=10
        )

        # Create the search entry.
        self.search_entry = Gtk.SearchEntry(
            placeholder_text="Search data...",
            margin_start=10,
            margin_end=10
        )

        # FIX: Use a Gio.ListStore with the custom DataItem type.
        data_model = Gio.ListStore.new(item_type=DataItem)
        
        # Add some dummy data to the store.
        data_model.append(DataItem("Item 1", "Category A", 100))
        data_model.append(DataItem("Item 2", "Category B", 250))
        data_model.append(DataItem("Item 3", "Category A", 50))
        data_model.append(DataItem("Item 4", "Category C", 320))
        data_model.append(DataItem("Item 5", "Category B", 150))
        
        # FIX: The filter is now a method of the class, passed directly to Gtk.CustomFilter.
        search_filter = Gtk.CustomFilter.new(lambda item: self.filter_func(item, self.search_entry.get_text()))
        
        # Create a FilterListModel from the data model and the filter.
        filtered_model = Gtk.FilterListModel.new(data_model, search_filter)
        
        # Connect the search entry to the filter.
        # FIX: Remove the 'None' argument as Gtk.Filter.changed() does not accept it.
        self.search_entry.connect("search-changed", lambda entry: search_filter.changed())

        # Create factories for each column to display the correct property.
        name_factory = Gtk.SignalListItemFactory()
        def name_setup(factory, list_item):
            list_item.set_child(Gtk.Label())
        def name_bind(factory, list_item):
            label = list_item.get_child()
            data_item = list_item.get_item()
            if data_item:
                label.set_label(data_item.name)
        name_factory.connect("setup", name_setup)
        name_factory.connect("bind", name_bind)

        category_factory = Gtk.SignalListItemFactory()
        def category_setup(factory, list_item):
            list_item.set_child(Gtk.Label())
        def category_bind(factory, list_item):
            label = list_item.get_child()
            data_item = list_item.get_item()
            if data_item:
                label.set_label(data_item.category)
        category_factory.connect("setup", category_setup)
        category_factory.connect("bind", category_bind)

        value_factory = Gtk.SignalListItemFactory()
        def value_setup(factory, list_item):
            list_item.set_child(Gtk.Label())
        def value_bind(factory, list_item):
            label = list_item.get_child()
            data_item = list_item.get_item()
            if data_item:
                label.set_label(str(data_item.value))
        value_factory.connect("setup", value_setup)
        value_factory.connect("bind", value_bind)

        # Use the filtered model for the column view.
        column_view = Gtk.ColumnView.new(Gtk.SingleSelection.new(filtered_model))
        column_view.set_vexpand(True)
        column_view.set_hexpand(True)

        name_column = Gtk.ColumnViewColumn.new("Name", name_factory)
        category_column = Gtk.ColumnViewColumn.new("Category", category_factory)
        value_column = Gtk.ColumnViewColumn.new("Value", value_factory)

        column_view.append_column(name_column)
        column_view.append_column(category_column)
        column_view.append_column(value_column)
        
        # Add a ScrolledWindow to the column view for scrolling.
        scrolled_window = Gtk.ScrolledWindow()
        scrolled_window.set_child(column_view)

        data_box.append(self.search_entry)
        data_box.append(scrolled_window)
        
        self.content_stack.add_titled_with_icon(data_box, "data", "Data", "document-edit-symbolic")

        # Create the new view switcher and its container box.
        view_switcher = Adw.ViewSwitcher(stack=self.content_stack, halign=Gtk.Align.CENTER)
        view_switcher_box = Gtk.Box(
            orientation=Gtk.Orientation.HORIZONTAL,
            halign=Gtk.Align.CENTER,
            margin_top=10,
            margin_bottom=10
        )
        view_switcher_box.append(view_switcher)
        
        # Create a top-level box to hold the new dropdown and the navigation view.
        main_window_box = Gtk.Box(
            orientation=Gtk.Orientation.VERTICAL,
            vexpand=True,
            hexpand=True
        )
        main_window_box.append(view_switcher_box)
        # The device_selection_box is no longer appended here.
        main_window_box.append(self.nav_view)

        # Create the main application window.
        self.win = Adw.ApplicationWindow(application=app)
        self.win.set_default_size(800, 600)  # Set a default size for the window.
        self.win.set_content(main_window_box) # Set the top-level box as the content.

    def filter_func(self, item, search_text):
        """
        The callback function used by the Gtk.CustomFilter.
        Returns True if the item should be displayed, False otherwise.
        """
        # FIX: Check for item being None before accessing its properties.
        if item is None:
            return False

        if not search_text:
            return True
        
        search_text_lower = search_text.lower()
        
        # Check if the search text is in any of the item's properties.
        if search_text_lower in item.name.lower():
            return True
        if search_text_lower in item.category.lower():
            return True
        # Convert the integer value to a string for comparison.
        if search_text_lower in str(item.value).lower():
            return True
            
        return False

# Run the application.
if __name__ == "__main__":
    app = MyApp()
    app.run(sys.argv)



# Copyright (C) 2017-2026 Arun Sivaraman <arunsivaramanneo@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

import sys
import gi
gi.require_version('Gtk','4.0')
gi.require_version(namespace='Adw', version='1')

from gi.repository import Gtk,GdkPixbuf,GObject,Gio,Adw,GLib
from Common import ExpandDataObject, setup_expander,bind_expander,setup,bind1,add_tree_node, ExpandDataObject2,add_tree_node2,bind2,bind3,bind4,fetchContentsFromCommand
from itertools import islice

Adw.init()

import const
import Filenames
import subprocess
from Common import copyContentsFromFile, getGpuImage,create_scrollbar, getRamInGb,createSubTab,getDriverVersion,getDeviceSize, setMargin,fetchContentsFromCommand,getVulkanVersion,createMainFile,getLogo

def VulkanVideo(videoTab):



    tab_spinners = {}
    tab_content_containers = {}
    tabs_loading_count = {"count": 0}

    def hide_spinner_for_tab(tab_name):
        if tab_name in tab_spinners and tab_name in tab_content_containers:
            def update_ui():
                spinner = tab_spinners[tab_name]
                content_box, spinner_box = tab_content_containers[tab_name]
                spinner.set_spinning(False)
                spinner_box.set_visible(False)
                content_box.set_visible(True)
                tabs_loading_count["count"] -= 1
                return False
            GLib.idle_add(update_ui)

    def vProfiles(gpu):
        from concurrent.futures import ThreadPoolExecutor
        def run_cmd(cmd):
            return fetchContentsFromCommand(cmd)

        with ThreadPoolExecutor(max_workers=2) as pool:
            f1 = pool.submit(run_cmd, r"vulkaninfo | awk '/GPU%d/{flag=1;next}/GPU%d/{flag=0}flag' | awk '/Video Profiles/{flag=1;}/Display.*/{flag=0}flag' | awk /./ | awk '!/===/' "%(gpu,gpu+1))
            f2 = pool.submit(run_cmd, r"vulkaninfo | awk '/GPU%d/{flag=1;next}/GPU%d/{flag=0}flag' | awk '/Video Profiles/{flag=1;next}flag' | awk /./ | awk '!/===/' | awk '{gsub(/\(.*/, 'True');print}' | uniq "%(gpu,gpu+1))
        
        vulkanVideoProfiles = f1.result()
        vulkanVideoProfilesUniq = f2.result()
        
        vulkanVideoProfilesCount = []
        for i in range(len(vulkanVideoProfilesUniq)):
            count = 0
            for profile in vulkanVideoProfiles:
                if vulkanVideoProfilesUniq[i] in profile:
                    count = count + 1
            vulkanVideoProfilesCount.append(count)

        vulkanVideoProfilesUniq.append("")

        def update_ui():
            vkVideoProfilesTab_Store.remove_all()
            j = 0; k = 0 ; group = None
            toprow = None
            for line in vulkanVideoProfiles:
                if 'Video Profiles' in line:
                    toprow = ExpandDataObject(line.replace('count =',''),"")
                    continue
                if not toprow:
                    continue
                if vulkanVideoProfilesUniq[j] in line and j <= len(vulkanVideoProfilesUniq) - 2:
                    iter1 = ExpandDataObject(vulkanVideoProfilesUniq[j].replace("placeholder =","")+": %d" %vulkanVideoProfilesCount[j],"")
                    toprow.children.append(iter1)
                    iter2 = ExpandDataObject(line.replace("placeholder =",""),"")
                    iter1.children.append(iter2)
                    j = j + 1
                else:
                    iter2 = ExpandDataObject(line.replace("placeholder =",""),"")
                    # Try to append to the last iter1, or toprow if not exists
                    if len(toprow.children) > 0:
                        toprow.children[-1].children.append(iter2)
                    else:
                        toprow.children.append(iter2)
                    continue

            if toprow:
                vkVideoProfilesTab_Store.append(toprow)
            hide_spinner_for_tab("Video Profiles")
            return False
        GLib.idle_add(update_ui)

    def vDefinitions(gpu):
        from concurrent.futures import ThreadPoolExecutor
        def run_cmd(cmd):
            return fetchContentsFromCommand(cmd)

        with ThreadPoolExecutor(max_workers=4) as pool:
            f1 = pool.submit(run_cmd, "vulkaninfo --show-video-props | awk '/GPU%d/{flag=1;next}/GPU%d/{flag=0}flag' | awk '/Video Profiles/{flag=1;next}flag' | awk '/Video Profile Definition/{flag=1}/Video Profile Capabilities/{flag=0}flag'| awk /./ | awk '!/---|===/' "%(gpu,gpu+1))
            f2 = pool.submit(run_cmd, "vulkaninfo --show-video-props | awk '/GPU%d/{flag=1;next}/GPU%d/{flag=0}flag' | awk '/Video Profiles/{flag=1;next}flag' | awk '/Video Profile Definition/{flag=1}/Video Profile Capabilities/{flag=0}flag'| awk /./ | awk '!/---|===/' | awk '{gsub(/[=,:].*/,'True')}1' "%(gpu,gpu+1))
            f3 = pool.submit(run_cmd, "vulkaninfo --show-video-props | awk '/GPU%d/{flag=1;next}/GPU%d/{flag=0}flag' | awk '/Video Profiles/{flag=1;next}flag' | awk '/Video Profile Definition/{flag=1}/Video Profile Capabilities/{flag=0}flag'| awk /./ | awk '!/---|===/' | grep -o [=].* | grep -o ' .*' "%(gpu,gpu+1))
            f4 = pool.submit(run_cmd, "vulkaninfo | awk '/GPU%d/{flag=1;next}/GPU%d/{flag=0}flag' | awk '/Video Profiles/{flag=1;}flag' | awk /./ | awk '!/===/' | grep placeholder "%(gpu,gpu+1))

        vulkanVideoProfileDefinition = f1.result()
        vulkanVideoProfileDefinitionLHS = f2.result()
        vulkanVideoProfileDefinitionRHS = f3.result()
        vulkanVideoProfiles = f4.result()
        vulkanVideoProfiles.append("")

        def update_ui():
            vkVideoProfileDefinitionTab_Store.remove_all(); print("update_ui running for Profile Definition"); print("LENGTH:", len(vulkanVideoProfileDefinition))
            group = None
            j = 0;i =0 ;k = 0
            iter1 = None
            for line in vulkanVideoProfileDefinition:
                if '=' in line and "Video Profile Definition" not in line:
                    rhs = vulkanVideoProfileDefinitionRHS[k].replace('count =','') if k < len(vulkanVideoProfileDefinitionRHS) else ""
                    k = k + 1
                else:
                    rhs = ""
                if "Video Profile Definition" in line:
                    if group == None:
                        if j < len(vulkanVideoProfiles):
                            iter1 = ExpandDataObject((vulkanVideoProfiles[j]).replace("placeholder =",""),"")
                    else:
                        if iter1:
                            vkVideoProfileDefinitionTab_Store.append(iter1); print("Appended iter1 to Profile Definition Store")
                        if j < len(vulkanVideoProfiles):
                            iter1 = ExpandDataObject((vulkanVideoProfiles[j]).replace("placeholder =",""),"")
                    j = j + 1
                    i = i + 1
                    group = line
                    continue
                
                if not iter1 or i >= len(vulkanVideoProfileDefinitionLHS):
                    continue
                    
                lhs_val = vulkanVideoProfileDefinitionLHS[i].strip("\t")
                if "\t" in line and "\t\t" not in line:
                    iter2 = ExpandDataObject(lhs_val,rhs)
                    iter1.children.append(iter2)
                    i = i + 1
                    continue
                elif "\t\t" in line and "\t\t\t" not in line:
                    iter3 = ExpandDataObject(lhs_val,rhs)
                    iter2.children.append(iter3)
                    i = i + 1
                    continue
                elif "\t\t\t" in line and "\t\t\t\t" not in line:
                    iter4 = ExpandDataObject(lhs_val,rhs)
                    iter3.children.append(iter4)
                    i = i + 1
                    continue            
                elif "\t\t\t\t" in line and "\t\t\t\t\t" not in line:
                    iter5 = ExpandDataObject(lhs_val,rhs)
                    iter4.children.append(iter5)
                    i = i + 1
                    continue    
                else:
                    iter6 = ExpandDataObject(lhs_val,rhs)
                    iter5.children.append(iter6)
                    i = i + 1
                    
            if iter1:
                vkVideoProfileDefinitionTab_Store.append(iter1); print("Appended iter1 to Profile Definition Store")
            hide_spinner_for_tab("Profile Definition")
            return False
        GLib.idle_add(update_ui)
    def vCapabitlies(gpu):
        from concurrent.futures import ThreadPoolExecutor
        def run_cmd(cmd):
            return fetchContentsFromCommand(cmd)

        with ThreadPoolExecutor(max_workers=4) as pool:
            f1 = pool.submit(run_cmd, "vulkaninfo --show-video-props | awk '/GPU%d/{flag=1;next}/GPU%d/{flag=0}flag' | awk '/Video Profiles/{flag=1;next}flag' | awk '/Video Profile Capabilities/{flag=1}/Video Formats/{flag=0}flag'| awk /./ | awk '!/---|===/' "%(gpu,gpu+1))
            f2 = pool.submit(run_cmd, "vulkaninfo --show-video-props | awk '/GPU%d/{flag=1;next}/GPU%d/{flag=0}flag' | awk '/Video Profiles/{flag=1;next}flag' | awk '/Video Profile Capabilities/{flag=1}/Video Formats/{flag=0}flag'| awk /./ | awk '!/---|===/' | awk '{gsub(/[=,:].*/,'True')}1' "%(gpu,gpu+1))
            f3 = pool.submit(run_cmd, "vulkaninfo --show-video-props | awk '/GPU%d/{flag=1;next}/GPU%d/{flag=0}flag' | awk '/Video Profiles/{flag=1;next}flag' | awk '/Video Profile Capabilities/{flag=1}/Video Formats/{flag=0}flag'| awk /./ | awk '!/---|===/' | grep -o [=].* | grep -o ' .*' "%(gpu,gpu+1))
            f4 = pool.submit(run_cmd, "vulkaninfo | awk '/GPU%d/{flag=1;next}/GPU%d/{flag=0}flag' | awk '/Video Profiles/{flag=1;}flag' | awk /./ | awk '!/===/' | grep placeholder "%(gpu,gpu+1))

        vulkanVideoCapabilities = f1.result()
        vulkanVideoCapabilitiesLHS = f2.result()
        vulkanVideoCapabilitiesRHS = f3.result()
        vulkanVideoProfiles = f4.result()
        vulkanVideoProfiles.append("")

        def update_ui():
            vkVideoCapabilitiesTab_Store.remove_all()
            group = None
            j = 0;i = 0 ; k = 0
            iter1 = None
            for line in vulkanVideoCapabilities:
                if '=' in line:
                    rhs = vulkanVideoCapabilitiesRHS[k].replace('count =','') if k < len(vulkanVideoCapabilitiesRHS) else ""
                    k = k + 1
                else:
                    rhs = ""
                if "Video Profile Capabilities" in line:
                    if group == None:
                        if j < len(vulkanVideoProfiles):
                            iter1 = ExpandDataObject((vulkanVideoProfiles[j]).replace("placeholder =",""),"")
                    else:
                        if iter1:
                            vkVideoCapabilitiesTab_Store.append(iter1)
                        if j < len(vulkanVideoProfiles):
                            iter1 = ExpandDataObject((vulkanVideoProfiles[j]).replace("placeholder =",""),"")
                    j = j + 1
                    i = i + 1
                    group = line
                    continue
                
                if not iter1 or i >= len(vulkanVideoCapabilitiesLHS):
                    continue
                    
                lhs_val = vulkanVideoCapabilitiesLHS[i].strip("\t")
                if "\t" in line and "\t\t" not in line:
                    iter2 = ExpandDataObject(lhs_val,rhs)
                    iter1.children.append(iter2)
                    i = i + 1
                    continue
                elif "\t\t" in line and "\t\t\t" not in line:
                    iter3 = ExpandDataObject(lhs_val,rhs)
                    iter2.children.append(iter3)
                    i = i + 1
                    continue
                elif "\t\t\t" in line and "\t\t\t\t" not in line:
                    iter4 = ExpandDataObject(lhs_val,rhs)
                    iter3.children.append(iter4)
                    i = i + 1
                    continue            
                elif "\t\t\t\t" in line and "\t\t\t\t\t" not in line:
                    iter5 = ExpandDataObject(lhs_val,rhs)
                    iter4.children.append(iter5)
                    i = i + 1
                    continue    
                else:
                    iter6 = ExpandDataObject(lhs_val,rhs)
                    iter5.children.append(iter6)
                    i = i + 1
                    
            if iter1:
                vkVideoCapabilitiesTab_Store.append(iter1)
            hide_spinner_for_tab("Video Capabilities")
            return False
        GLib.idle_add(update_ui)
    def vFormats(gpu):
        from concurrent.futures import ThreadPoolExecutor
        def run_cmd(cmd):
            return fetchContentsFromCommand(cmd)

        with ThreadPoolExecutor(max_workers=4) as pool:
            f1 = pool.submit(run_cmd, "vulkaninfo --show-video-props | awk '/GPU%d/{flag=1;next}/GPU%d/{flag=0}flag' | awk '/Video Profiles/{flag=1;next}flag' | awk '/Video Formats:/{flag=1}/Display.*/{flag=0}flag'| awk '/./' | awk '!/---|===/' "%(gpu,gpu+1))
            f2 = pool.submit(run_cmd, "vulkaninfo --show-video-props | awk '/GPU%d/{flag=1;next}/GPU%d/{flag=0}flag' | awk '/Video Profiles/{flag=1;next}flag' | awk '/Video Formats:/{flag=1}/Display.*/{flag=0}flag'| awk '/./' | awk '!/---|===/' | awk '{gsub(/[=,:].*/,'True')}1' "%(gpu,gpu+1))
            f3 = pool.submit(run_cmd, "vulkaninfo --show-video-props | awk '/GPU%d/{flag=1;next}/GPU%d/{flag=0}flag' | awk '/Video Profiles/{flag=1;next}flag' | awk '/Video Formats:/{flag=1}/Display.*/{flag=0}flag'| awk '/./' | awk '!/---|===/' | grep -o [=].* | grep -o ' .*' "%(gpu,gpu+1))
            f4 = pool.submit(run_cmd, "vulkaninfo | awk '/GPU%d/{flag=1;next}/GPU%d/{flag=0}flag' | awk '/Video Profiles/{flag=1;}flag' | awk /./ | awk '!/===/' | grep placeholder "%(gpu,gpu+1))

        vulkanVideoFormats = f1.result()
        vulkanVideoFormatsLHS = f2.result()
        vulkanVideoFormatsRHS = f3.result()
        vulkanVideoProfiles = f4.result()
        vulkanVideoProfiles.append("")

        def update_ui():
            vkVideoFormatsTab_Store.remove_all()
            group = None
            j = 1;i = 0; k = 0;
            toprow = None
            if len(vulkanVideoProfiles) > 0:
                toprow = ExpandDataObject(vulkanVideoProfiles[0].replace("placeholder = ",""),"")
                
            for line in vulkanVideoFormats:
                if '=' in line:
                    rhs = vulkanVideoFormatsRHS[k].replace('count =','') if k < len(vulkanVideoFormatsRHS) else ""
                    k = k + 1
                else:
                    rhs = ""
                    
                if not toprow:
                    continue
                    
                if j < len(vulkanVideoProfiles) and line.strip(":") in vulkanVideoProfiles[j].replace("placeholder = ",""):
                    vkVideoFormatsTab_Store.append(toprow)
                    toprow = ExpandDataObject(vulkanVideoProfiles[j].replace("placeholder = ",""),"")
                    j = j + 1
                    i = i + 1
                    group = line
                    continue
                    
                if i >= len(vulkanVideoFormatsLHS):
                    continue
                    
                lhs_val = vulkanVideoFormatsLHS[i].strip("\t")
                if "Video Formats" in line:
                    iter1 = ExpandDataObject(lhs_val,rhs)
                    toprow.children.append(iter1)
                    i = i + 1
                    continue
                elif "\t" in line and "\t\t" not in line:
                    iter2 = ExpandDataObject(lhs_val,rhs)
                    iter1.children.append(iter2)
                    i = i + 1
                    continue
                elif "\t\t" in line and "\t\t\t" not in line:
                    iter3 = ExpandDataObject(lhs_val,rhs)
                    iter2.children.append(iter3)
                    i = i + 1
                    continue
                elif "\t\t\t" in line and "\t\t\t\t" not in line:
                    iter4 = ExpandDataObject(lhs_val,rhs)
                    iter3.children.append(iter4)
                    i = i + 1
                    continue            
                elif "\t\t\t\t" in line and "\t\t\t\t\t" not in line:
                    iter5 = ExpandDataObject(lhs_val,rhs)
                    iter4.children.append(iter5)
                    i = i + 1
                    continue    
                else:
                    iter6 = ExpandDataObject(lhs_val,rhs)
                    iter5.children.append(iter6)
                    i = i + 1
                    
            if toprow:
                vkVideoFormatsTab_Store.append(toprow)
            hide_spinner_for_tab("Video Formats")
            return False
        GLib.idle_add(update_ui)

    def on_gpu_dropdown_changed(combo,dummy):
        text = combo.props.selected_item
        for gpu in range(len(gpu_list)):
            if gpu_list[gpu] in text.get_string():
                tabs_loading_count["count"] = 4
                for tab in tab_spinners:
                    tab_spinners[tab].start()
                    tab_spinners[tab].set_visible(True)
                    tab_content_containers[tab][1].set_visible(True)
                    tab_content_containers[tab][0].set_visible(False)
                    
                import threading
                def load_all_bg(selected_gpu):
                    try:
                        vProfiles(selected_gpu)
                        vDefinitions(selected_gpu)
                        vCapabitlies(selected_gpu)
                        vFormats(selected_gpu)
                    except Exception as e:
                        import traceback
                        traceback.print_exc()
                threading.Thread(target=load_all_bg, args=(gpu,), daemon=True).start()

                gpu_image = getGpuImage(gpu_list[gpu])
                image_renderer.set_pixbuf(gpu_image)
                break





    h_box = Gtk.Box.new(Gtk.Orientation.HORIZONTAL, 10)
    h_box.set_halign(Gtk.Align.CENTER)
    
    # Add a top margin to create space between the header bar and the content
    h_box.set_margin_top(10)
    h_box.set_margin_bottom(10)
    h_box.set_margin_start(10)

    # Create the label for the devices
    label = Gtk.Label.new("Available Device(s) :")
    h_box.append(label)

    # Create a list with some dummy device names
    gpu_list = fetchContentsFromCommand(Filenames.fetch_vulkaninfo_ouput_command+Filenames.fetch_device_name_command)
    
    # Create the gpu_dropdown, populated from the list of strings
    gpu_Dropdown = Gtk.DropDown()
    gpu_Dropdown.add_css_class(css_class="card")
    gpu_Dropdown_list = Gtk.StringList()
    gpu_Dropdown.set_model(gpu_Dropdown_list)
    gpu_Dropdown.set_margin_start(10)
    h_box.append(gpu_Dropdown)
    
    unique_gpus = []
    for i in range(len(gpu_list)):
        # Re-use the cached vulkaninfo file — avoids spawning vulkaninfo once per GPU
        check_cmd = (
            f"cat {Filenames.vulkaninfo_output_file} "
            f"| awk '/GPU{i}/{{flag=1;next}}/GPU{i+1}/{{flag=0}}flag' "
            f"| grep 'Video Profiles'"
        )
        if fetchContentsFromCommand(check_cmd):
            if gpu_list[i] not in unique_gpus:
                unique_gpus.append(gpu_list[i])
                gpu_Dropdown_list.append(gpu_list[i])
    # Add the horizontal box to the main vertical box
    videoTab.append(h_box)
    
    split_view_container = Gtk.Box.new(Gtk.Orientation.VERTICAL, 0)
    split_view_container.set_vexpand(True)
    split_view_container.set_hexpand(True)
    
    # Create a new NavigationSplitView
    split_view = Adw.NavigationSplitView.new()
    split_view.set_vexpand(True)
    split_view.set_hexpand(True)

    # Create the sidebar content using a Gtk.ListBox for tabs
    sidebar_listbox = Gtk.ListBox.new()
    sidebar_listbox.set_selection_mode(Gtk.SelectionMode.SINGLE)
    sidebar_listbox.set_vexpand(True)
    sidebar_listbox.add_css_class(css_class="boxed-list")
    sidebar_listbox.add_css_class(css_class="sidebar")
    


#    sidebar_listbox.add_css_class(css_class="frame")
    # Set the show-separators property to False
    sidebar_listbox.set_show_separators(True)
    
    # Add the tabs to the sidebar ListBox
    tabs = [
        "Video Profiles", "Profile Definition", "Video Capabilities", "Video Formats"
    ]

    content_stack = Gtk.Stack.new()
    
    for tab_name in tabs:
        row = Gtk.ListBoxRow()
        label = Gtk.Label.new(tab_name)
        # Add padding to the label inside the row
        label.set_margin_top(10)
        label.set_margin_bottom(10)
        row.set_child(label)
        sidebar_listbox.append(row)


        # Create a box for the content of this specific tab
        content_box = Gtk.Box.new(Gtk.Orientation.VERTICAL, 10)
        
        spinner_box = Gtk.Box.new(Gtk.Orientation.VERTICAL, 12)
        spinner_box.set_valign(Gtk.Align.CENTER)
        spinner_box.set_halign(Gtk.Align.CENTER)
        spinner_box.set_vexpand(True)
        spinner = Gtk.Spinner()
        spinner.set_size_request(36, 36)
        lbl = Gtk.Label(label=f"Loading {tab_name}…")
        lbl.add_css_class("dim-label")
        spinner_box.append(spinner)
        spinner_box.append(lbl)
        
        tab_spinners[tab_name] = spinner
        tab_content_containers[tab_name] = (content_box, spinner_box)
        
        tab_wrapper = Gtk.Box.new(Gtk.Orientation.VERTICAL, 0)
        content_box.set_visible(False)
        tab_wrapper.append(spinner_box)
        tab_wrapper.append(content_box)
        
        if tab_name == "Video Profiles":
 # --------------------- Vulkan Video Profiles ---------------------------------------------------------------------
            vkVideoProfilesColumnView = Gtk.ColumnView()
            vkVideoProfilesColumnView.props.show_row_separators = True
            vkVideoProfilesColumnView.props.show_column_separators = False

            factory_vkVideoProfiles = Gtk.SignalListItemFactory()
            factory_vkVideoProfiles.connect("setup",setup_expander)
            factory_vkVideoProfiles.connect("bind",bind_expander)

            factory_vkVideoProfiles_value = Gtk.SignalListItemFactory()
            factory_vkVideoProfiles_value.connect("setup",setup)
            factory_vkVideoProfiles_value.connect("bind",bind1)

            vkVideoProfilesSelection = Gtk.SingleSelection()
            vkVideoProfilesTab_Store = Gio.ListStore.new(ExpandDataObject)

            vkVideoModel = Gtk.TreeListModel.new(vkVideoProfilesTab_Store,False,True,add_tree_node)
            vkVideoProfilesSelection.set_model(vkVideoModel)

            vkVideoProfilesColumnView.set_model(vkVideoProfilesSelection)

            vkVideoProfilesColumnLhs = Gtk.ColumnViewColumn.new("Details",factory_vkVideoProfiles)
            vkVideoProfilesColumnLhs.set_resizable(True)
            vkVideoProfilesColumnRhs = Gtk.ColumnViewColumn.new("",factory_vkVideoProfiles_value)
            vkVideoProfilesColumnRhs.set_expand(True)

            vkVideoProfilesColumnView.append_column(vkVideoProfilesColumnLhs)
            vkVideoProfilesColumnView.append_column(vkVideoProfilesColumnRhs)

            vkVideoProfilesScrollbar = create_scrollbar(vkVideoProfilesColumnView)
            content_box.append(vkVideoProfilesScrollbar)
        
        elif tab_name == "Profile Definition":
        # --------------------- Vulkan Video Profile Definition ---------------------------------------------------------------------

            vkVideoProfileDefinitionColumnView = Gtk.ColumnView()
            vkVideoProfileDefinitionColumnView.props.show_row_separators = True
            vkVideoProfileDefinitionColumnView.props.show_column_separators = False

            factory_vkVideoProfileDefinition = Gtk.SignalListItemFactory()
            factory_vkVideoProfileDefinition.connect("setup",setup_expander)
            factory_vkVideoProfileDefinition.connect("bind",bind_expander)

            factory_vkVideoProfileDefinition_value = Gtk.SignalListItemFactory()
            factory_vkVideoProfileDefinition_value.connect("setup",setup)
            factory_vkVideoProfileDefinition_value.connect("bind",bind1)

            vkVideoProfileDefinitionSelection = Gtk.SingleSelection()
            vkVideoProfileDefinitionTab_Store = Gio.ListStore.new(ExpandDataObject)

            vkVideoModel = Gtk.TreeListModel.new(vkVideoProfileDefinitionTab_Store,False,True,add_tree_node)
            vkVideoProfileDefinitionSelection.set_model(vkVideoModel)

            vkVideoProfileDefinitionColumnView.set_model(vkVideoProfileDefinitionSelection)

            vkVideoProfileDefinitionColumnLhs = Gtk.ColumnViewColumn.new("Details",factory_vkVideoProfileDefinition)
            vkVideoProfileDefinitionColumnLhs.set_resizable(True)
            vkVideoProfileDefinitionColumnRhs = Gtk.ColumnViewColumn.new("Value",factory_vkVideoProfileDefinition_value)
            vkVideoProfileDefinitionColumnRhs.set_expand(True)

            vkVideoProfileDefinitionColumnView.append_column(vkVideoProfileDefinitionColumnLhs)
            vkVideoProfileDefinitionColumnView.append_column(vkVideoProfileDefinitionColumnRhs)

            vkVideoProfileDefinitionScrollbar = create_scrollbar(vkVideoProfileDefinitionColumnView)
            content_box.append(vkVideoProfileDefinitionScrollbar)

        elif tab_name == "Video Capabilities":
        # -------------------- Vulkan Video Capabilities -------------------------------------------------------------------------------

            vkVideoCapabilitiesColumnView = Gtk.ColumnView()
            vkVideoCapabilitiesColumnView.props.show_row_separators = True
            vkVideoCapabilitiesColumnView.props.show_column_separators = False

            factory_vkVideoCapabilities = Gtk.SignalListItemFactory()
            factory_vkVideoCapabilities.connect("setup",setup_expander)
            factory_vkVideoCapabilities.connect("bind",bind_expander)

            factory_vkVideoCapabilities_value = Gtk.SignalListItemFactory()
            factory_vkVideoCapabilities_value.connect("setup",setup)
            factory_vkVideoCapabilities_value.connect("bind",bind1)

            vkVideoCapabilitiesSelection = Gtk.SingleSelection()
            vkVideoCapabilitiesTab_Store = Gio.ListStore.new(ExpandDataObject)

            vkVideoModel = Gtk.TreeListModel.new(vkVideoCapabilitiesTab_Store,False,True,add_tree_node)
            vkVideoCapabilitiesSelection.set_model(vkVideoModel)

            vkVideoCapabilitiesColumnView.set_model(vkVideoCapabilitiesSelection)

            vkVideoCapabilitiesColumnLhs = Gtk.ColumnViewColumn.new("Details",factory_vkVideoCapabilities)
            vkVideoCapabilitiesColumnLhs.set_resizable(True)
            vkVideoCapabilitiesColumnRhs = Gtk.ColumnViewColumn.new("Value",factory_vkVideoCapabilities_value)
            vkVideoCapabilitiesColumnRhs.set_expand(True)

            vkVideoCapabilitiesColumnView.append_column(vkVideoCapabilitiesColumnLhs)
            vkVideoCapabilitiesColumnView.append_column(vkVideoCapabilitiesColumnRhs)

            vkVideoCapabilitiesScrollbar = create_scrollbar(vkVideoCapabilitiesColumnView)

            content_box.append(vkVideoCapabilitiesScrollbar)
        elif tab_name == "Video Formats":
        
            # -------------------- Vulkan Video Formats -------------------------------------------------------------------------------

            vkVideoFormatsColumnView = Gtk.ColumnView()
            vkVideoFormatsColumnView.props.show_row_separators = True
            vkVideoFormatsColumnView.props.show_column_separators = False

            factory_vkVideoFormats = Gtk.SignalListItemFactory()
            factory_vkVideoFormats.connect("setup",setup_expander)
            factory_vkVideoFormats.connect("bind",bind_expander)

            factory_vkVideoFormats_value = Gtk.SignalListItemFactory()
            factory_vkVideoFormats_value.connect("setup",setup)
            factory_vkVideoFormats_value.connect("bind",bind1)

            vkVideoFormatsSelection = Gtk.SingleSelection()
            vkVideoFormatsTab_Store = Gio.ListStore.new(ExpandDataObject)

            vkVideoModel = Gtk.TreeListModel.new(vkVideoFormatsTab_Store,False,True,add_tree_node)
            vkVideoFormatsSelection.set_model(vkVideoModel)

            vkVideoFormatsColumnView.set_model(vkVideoFormatsSelection)

            vkVideoFormatsColumnLhs = Gtk.ColumnViewColumn.new("Details",factory_vkVideoFormats)
            vkVideoFormatsColumnLhs.set_resizable(True)
            vkVideoFormatsColumnRhs = Gtk.ColumnViewColumn.new("Value",factory_vkVideoFormats_value)
            vkVideoFormatsColumnRhs.set_expand(True)

            vkVideoFormatsColumnView.append_column(vkVideoFormatsColumnLhs)
            vkVideoFormatsColumnView.append_column(vkVideoFormatsColumnRhs)

            vkVideoFormatsScrollbar = create_scrollbar(vkVideoFormatsColumnView)
            content_box.append(vkVideoFormatsScrollbar)

        content_stack.add_titled(tab_wrapper, tab_name.replace(" ", "-").lower(), tab_name)

    gpu_Dropdown.connect("notify::selected-item", on_gpu_dropdown_changed)

    gpu_image = Gtk.Image()
    gpu_image = GdkPixbuf.Pixbuf.new_from_file_at_size(const.APP_LOGO_PNG, 100, 100)
    image_renderer = Gtk.Picture.new_for_pixbuf(gpu_image)
    gpu_Dropdown.set_margin_end(10)
    h_box.append(image_renderer)

    on_gpu_dropdown_changed(gpu_Dropdown,dummy=0)
    # Create a scrolled window for the sidebar list
    sidebar_scrolled_window = Gtk.ScrolledWindow.new()
    sidebar_scrolled_window.set_child(sidebar_listbox)

    # Connect the listbox signal to change the stack page
    def on_row_activated(listbox, row):
        # Get the label from the activated row to find the tab's name
        tab_label = row.get_child()
        tab_name = tab_label.get_label()
        
        # The name for the stack child is the tab name, with spaces replaced by dashes and lowercase
        child_name = tab_name.replace(" ", "-").lower()

        # Set the visible child of the stack using its name
        content_stack.set_visible_child_name(child_name)
        
    sidebar_listbox.connect("row-activated", on_row_activated)
    sidebar_listbox.select_row(sidebar_listbox.get_row_at_index(0))
#    content_stack.set_visible_child_name("System Information")
    # Set a minimum width for the sidebar


    # Create the Adw.NavigationPage wrappers for the sidebar and content
    sidebar_page = Adw.NavigationPage.new(sidebar_scrolled_window, "Sidebar")
    content_page = Adw.NavigationPage.new(content_stack, "Content")
    
    # Set the sidebar and content for the split view using the new pages
    split_view.set_sidebar(sidebar_page)
    split_view.set_content(content_page)
    
    # Add the split view to its new container
    split_view_container.append(split_view)
    
    # Add the split view container to the main vertical box
    videoTab.append(split_view_container)

    return videoTab


# Copyright (C) 2017-2026 Arun Sivaraman <arunsivaramanneo@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

import sys
import gi
import const
import subprocess
import Filenames
from pathlib import Path
gi.require_version('Gtk','4.0')
gi.require_version(namespace='Adw', version='1')

from gi.repository import Gtk,GdkPixbuf,Gdk,Gio,GObject,Adw,GLib,Pango
import cairo as _cairo


import os
import glob
import configparser
import math

# Removed CircularGauge - reverting to LevelBar

class Config:
    def __init__(self):
        self.config_dir = os.path.join(GLib.get_user_config_dir(), "gpu-viewer")
        self.config_path = os.path.join(self.config_dir, "config.ini")
        self.config = configparser.ConfigParser()
        self.load()

    def load(self):
        if not os.path.exists(self.config_dir):
            os.makedirs(self.config_dir)
        if os.path.exists(self.config_path):
            self.config.read(self.config_path)
        else:
            self.config["THEME"] = {"prefer_dark": "False"}
            self.save()

    def save(self):
        with open(self.config_path, "w") as f:
            self.config.write(f)

    def get_theme_preference(self):
        return self.config.getboolean("THEME", "prefer_dark", fallback=True)

    def set_theme_preference(self, prefer_dark):
        if "THEME" not in self.config:
            self.config["THEME"] = {}
        self.config["THEME"]["prefer_dark"] = str(prefer_dark)
        self.save()

Adw.init()

def get_gpu_stats(device_id, num_devices):
    """
    Fetches GPU stats (Memory Used, Memory Total, Temperature, Clock Current, Clock Max, Usage, Fan Speed, Power).
    Returns a dictionary: {'mem_used': int (MB), 'mem_total': int (MB), 'temp': int (C), 
                           'clock_current': int (MHz), 'clock_max': int (MHz), 'usage': int (%),
                           'fan_speed': int (%), 'power_usage': int (W)}
    or None if failed.
    """
    stats = {'mem_used': 0, 'mem_total': 0, 'temp': 0, 'clock_current': 0, 'clock_max': 0, 'usage': -1, 'fan_speed': -1, 'power_usage': -1}
    
    gpu_index = -1
    # Loop and check each card file to find the matching device_id
    for i in range(num_devices):
        device_file_path = f"/sys/class/drm/card{i}/device/device"
        if os.path.exists(device_file_path):
            try:
                with open(device_file_path, 'r') as f:
                    # Convert hex value from file to number and compare with device_id
                    sys_device_id = int(f.read().strip(), 16)
                    if sys_device_id == device_id:
                        gpu_index = i
                        break
            except (ValueError, OSError):
                continue

    if gpu_index == -1:
        return None

    # Try AMD / Intel (sysfs)
    # Use the found gpu_index
    card_path = f"/sys/class/drm/card{gpu_index}/device"
    if os.path.isdir(card_path):
        try:
            # AMD Memory
            if os.path.exists(f"{card_path}/mem_info_vram_used"):
                with open(f"{card_path}/mem_info_vram_used", 'r') as f:
                    stats['mem_used'] = int(f.read().strip()) // (1024 * 1024)
            
            if os.path.exists(f"{card_path}/mem_info_vram_total"):
                 with open(f"{card_path}/mem_info_vram_total", 'r') as f:
                    stats['mem_total'] = int(f.read().strip()) // (1024 * 1024)

            # Temperature, Fan, Power
            # Search for hwmon
            hwmon_pattern = f"{card_path}/hwmon/hwmon*"
            hwmons = glob.glob(hwmon_pattern)
            if hwmons:
                for hwmon in hwmons:
                    # Temperature
                    temp_input = f"{hwmon}/temp1_input"
                    if os.path.exists(temp_input):
                         with open(temp_input, 'r') as f:
                            stats['temp'] = int(f.read().strip()) // 1000
                    
                    # Fan Speed (PWM)
                    pwm_input = f"{hwmon}/pwm1"
                    pwm_max_input = f"{hwmon}/pwm1_max"
                    if os.path.exists(pwm_input):
                        with open(pwm_input, 'r') as f:
                            pwm_value = int(f.read().strip())
                            # PWM is usually 0-255, convert to percentage
                            pwm_max = 255
                            if os.path.exists(pwm_max_input):
                                try:
                                    with open(pwm_max_input, 'r') as f_max:
                                        pwm_max = int(f_max.read().strip())
                                except:
                                    pass
                            stats['fan_speed'] = int((pwm_value / pwm_max) * 100)
                    
                    # Power Usage
                    power_input = f"{hwmon}/power1_average"
                    if os.path.exists(power_input):
                        with open(power_input, 'r') as f:
                            # Power is in microwatts, convert to watts
                            stats['power_usage'] = int(f.read().strip()) // 1000000
            
            # GPU Usage (AMD specific)
            if os.path.exists(f"{card_path}/gpu_busy_percent"):
                with open(f"{card_path}/gpu_busy_percent", 'r') as f:
                    stats['usage'] = int(f.read().strip())
            
            # Clock (AMD specific usually)
            # Try pp_dpm_sclk first (shows list of clocks provided by driver, * marks current)
            if os.path.exists(f"{card_path}/pp_dpm_sclk"):
                with open(f"{card_path}/pp_dpm_sclk", 'r') as f:
                    lines = f.readlines()
                    max_clock = 0
                    current_clock = 0
                    for line in lines:
                        # Format: "0: 300Mhz *"
                        parts = line.split()
                        if len(parts) >= 2:
                            clock_str = parts[1].replace('Mhz', '').replace('MHz', '')
                            try:
                                clock_val = int(clock_str)
                                if clock_val > max_clock:
                                    max_clock = clock_val
                                if '*' in line:
                                    current_clock = clock_val
                            except ValueError:
                                pass
                    stats['clock_current'] = current_clock
                    stats['clock_max'] = max_clock
            
            # Fallback for clock: current freq from hwmon if pp_dpm_sclk fails or simpler card
            if stats['clock_current'] == 0 and hwmons:
                 for hwmon in hwmons:
                    freq_input = f"{hwmon}/freq1_input"
                    if os.path.exists(freq_input):
                         with open(freq_input, 'r') as f:
                            stats['clock_current'] = int(f.read().strip()) // (1000 * 1000) # Hz to MHz
                            break

            if stats['mem_total'] > 0: # minimal validation
                return stats
        except Exception as e:
            print(f"Error reading sysfs: {e}")
            pass

    return None



class MyGtk(Gtk.Window):
    def __init__(self, title):
        super(MyGtk, self).__init__(title=title)
        setting = Gtk.Settings.get_default()
        # Setting Theme
   #     if Path(Filenames.Materia_gtk_theme_folder).exists(): 
   #         setting.set_property("gtk-theme-name", "Materia-dark")
   #     elif Path(Filenames.Orchis_gtk_theme_folder).exists():
   #         setting.set_property("gtk-theme-name","Orchis-Compact")
   #     else:
   #         setting.set_property("gtk-theme-name","Adwaita")

        #Setting Font
    #    if Path(Filenames.Roboto_font_folder).exists():
    #        setting.set_property("gtk-font-name","Roboto-Black 12")
    #        setting.set_property("gtk-hint-font-metrics",True)

def create_tab(notebook,icon_url,icon_width,icon_height,aspect_ratio):
    tab = Gtk.Box(orientation=1,spacing=10)
 #   tab_icon = fetchImageFromUrl(icon_url,icon_width,icon_height,aspect_ratio)
    notebook.add_titled_with_icon(child=tab, name=icon_width, title=icon_width,icon_name=icon_url)
#    notebook.append_page(tab,Gtk.Picture.new_for_pixbuf(tab_icon))
    return tab

#getting Ram Details in GB

def setMargin(widget,start,top,bottom):
    widget.set_margin_start(start)
    widget.set_margin_top(top)
    widget.set_margin_bottom(bottom)

def getRamInGb(ram):
    ram1 = ram.split()
    return str("%.2f" %(float(ram1[0])/(1024*1024))) + " GB"

# Setting the Minimum Screen Size
def getScreenSize():
    display = Gdk.Display.get_default()
    monitors = display.get_monitors()
    for m in monitors:
        g = m.get_geometry()
    return g.width,g.height


# fetching the Images/Logos from the const File
def fetchImageFromUrl(imgUrl, iconWidth, iconHeight, aspectRatio):
    return GdkPixbuf.Pixbuf.new_from_file_at_scale(
        filename=imgUrl, width=iconWidth, height=iconHeight, preserve_aspect_ratio=aspectRatio)

# Copy the Contents of the file from a File to a List
def copyContentsFromFile(fileName):
    with open(fileName, "r") as file1:
        value = []
        for line in file1:
            value.append(line)
    return value

def fetchContentsFromCommand(command):
    process = subprocess.Popen(command,shell=True,stdout=subprocess.PIPE,universal_newlines=True)
    return process.communicate()[0].splitlines()

def createMainFile(filename,command):
    with open(filename,"w") as file:
        process = subprocess.Popen(command,shell=True,stdout=file,universal_newlines=True)
        process.communicate()

def on_light_action_actived(self, action, win,param=None):
    display = Gtk.Widget.get_display(win)
    provider = Gtk.CssProvider.new()
    fname = Gio.file_new_for_path('gtk_light.css')
    provider.load_from_file(fname)
    Gtk.StyleContext.add_provider_for_display(display, provider,
            Gtk.STYLE_PROVIDER_PRIORITY_USER)
    
        
def on_dark_action_actived(self, action,win, param=None):
    display = Gtk.Widget.get_display(win)
    provider = Gtk.CssProvider.new()
    fname = Gio.file_new_for_path('gtk_dark.css')
    provider.load_from_file(fname)
    Gtk.StyleContext.add_provider_for_display(display, provider,
            Gtk.STYLE_PROVIDER_PRIORITY_USER)
        

# setting up Sub Tabs in Vulkan

def createSubTab(Tab, notebook, label):
 #   Tab.set_border_width(10)
    notebook.append_page(Tab, Gtk.Label(label=label))
    Frame = Gtk.Frame()
    Tab.append(Frame)
    notebook.set_property('tab-pos',Gtk.PositionType.LEFT) 
    page = notebook.get_page(Tab)
    page.set_property("tab-expand",False)
    Grid = Gtk.Grid()
    Frame.set_child(Grid)
    return Grid


# adding Scrollbar to the Treeview

def create_scrollbar(widget):
    scrollbar = Gtk.ScrolledWindow()
    scrollbar.set_policy(Gtk.PolicyType.AUTOMATIC,Gtk.PolicyType.AUTOMATIC)
    scrollbar.set_vexpand(True)
    scrollbar.set_hexpand(True)
    scrollbar.set_visible(True)
    scrollbar.set_child(widget)
    return scrollbar


def colorTrueFalse(filename, text):
    with open(filename, "r") as file1:
        value = []
        fgColor = []
        for line in file1:
            if text in line:
                value.append("true")
                fgColor.append(const.COLOR1)
            else:
                value.append("false")
                fgColor.append(const.COLOR2)
    return fgColor, value

def getFormatValue(filename,Format):
    loop = 0
    value = []
    fgColor = []
    with open(filename,"r") as file:
        for line in file:
            for i,f in enumerate(Format):
                if "FEATURE" in line and i >= loop:
                    value.append("true")
                    fgColor.append(const.COLOR1)
                    loop = loop + 1
                    if ":" in f:
                        break
                if "None" in line and i >= loop:
                    value.append("false")
                    fgColor.append(const.COLOR2)
                    loop = loop + 1
                    if ":" in f:
                        break
    return fgColor, value


def getLinkButtonImg(img, link, toolTip):
    Logbutton = Gtk.LinkButton.new_with_label(link)
    Logbutton.set_child(Gtk.Picture.new_for_pixbuf(img))
    Logbutton.set_tooltip_text(toolTip)
    return Logbutton


def getVulkanVersion(value):
    majorVersion = int(value) >> 22
    minorVersion = int(value) >> 12 & 1023
    patchVersion = int(value) & 4095
    return "%d.%d.%d" % (majorVersion, minorVersion, patchVersion)


def getDriverVersion(value,i):
    if '4318' in value:
        majorVersion = (int(value[i]) >> 22) & 1023
        minorVersion = (int(value[i]) >> 14) & 255
        microVersion = (int(value[i]) >> 6) & 255
        nanoVersion = int(value[i]) & 63
        return "%d.%.2d.%.2d.%d" % (majorVersion, minorVersion, microVersion, nanoVersion)
    else:
        majorVersion = int(value[i]) >> 22
        minorVersion = int(value[i]) >> 12 & 1023
        microVersion = int(value[i]) & 4095
        return "%d.%d.%d" % (majorVersion, minorVersion, microVersion)

def setup(widget, item):
    """Setup the widget to show in the Gtk.Listview"""
    label = Gtk.Label()
    label.props.xalign = 0.0
    label.set_ellipsize(Pango.EllipsizeMode.END)
    item.set_child(label)


def bind(widget, item,column):
    """bind data from the store object to the widget"""
    label = item.get_child()
    obj = item.get_item()
    label.set_label(obj.column)
        
def createSearchEntry(store_filter):
    entry = Gtk.SearchEntry()
    entry.set_property("placeholder_text","Type here to filter.....")
    entry.connect("search-changed", refresh_filter, store_filter)
    return entry


def getDeviceSize(size):
    sizeMB = float(size) / (1024 * 1024 * 1024)
    if sizeMB < 1.0:
        sizeMB = str(format((sizeMB * 1024), '.2f')) + " MB"
    else:
        sizeMB = str(format(sizeMB, '.2f')) + " GB"
    return sizeMB

def appendLimitsRHS(filename, temp):
    LimitsRHS = []
    LimitRHSValue = []
    i = 0
    with open(filename, "r") as file1:
        for i, line in enumerate(file1):
            if i < len(temp):
                val = temp[i].strip()
                LimitsRHS.append(val)
                LimitRHSValue.append(val != "")
            else:
                LimitsRHS.append("")
                LimitRHSValue.append(False)
    return LimitsRHS, LimitRHSValue

def getGpuImage(line):
    if "Intel" in line and "Arc" not in line:
        gpu_image = fetchImageFromUrl(const.INTEL_LOGO_PNG, const.ICON_WIDTH, const.ICON_HEIGHT, True)
    elif "Intel" in line and "Arc" in line:
        gpu_image = fetchImageFromUrl(const.INTEL_ARC_LOGO_PNG, const.ICON_WIDTH, const.ICON_HEIGHT, True)
    elif "GTX" in line and "GeForce" in line:
        gpu_image = fetchImageFromUrl(const.NVIDIA_GTX_LOGO_PNG, const.ICON_WIDTH, const.ICON_HEIGHT, True)
    elif "RTX" in line and "GeForce" in line:
        gpu_image = fetchImageFromUrl(const.NVIDIA_RTX_LOGO_PNG, const.ICON_WIDTH, const.ICON_HEIGHT, True)
    elif "NVIDIA" in line:
        gpu_image = fetchImageFromUrl(const.NVIDIA_LOGO_PNG, const.ICON_WIDTH, const.ICON_HEIGHT, True)    
    elif "GeForce" in line and ("GTX" not in line or "RTX" not in line):
        gpu_image = fetchImageFromUrl(const.GEFORCE_PNG, const.ICON_WIDTH, const.ICON_HEIGHT, True)
    elif "CUDA" in line and ("GTX" not in line or "RTX" not in line):
        gpu_image = fetchImageFromUrl(const.CUDA_PNG, 100, const.ICON_HEIGHT, True)
    elif "Ryzen" in line and "AMD" in line:
        gpu_image = fetchImageFromUrl(const.AMDRYZEN_LOGO_PNG, const.ICON_WIDTH, const.ICON_HEIGHT, True)
    elif "Radeon" in line and "AMD" in line and "Pro" not in line:
        gpu_image = fetchImageFromUrl(const.AMDRADEON_LOGO_PNG, const.ICON_WIDTH, const.ICON_HEIGHT, True)
    elif "Radeon" in line and "AMD" in line and "Pro" in line:
        gpu_image = fetchImageFromUrl(const.AMD_RADEON_Pro_LOGO_PNG, const.ICON_WIDTH, const.ICON_HEIGHT, True)
    elif "AMD" in line or "ATI" in line and "Radeon" not in line:
        gpu_image = fetchImageFromUrl(const.AMD_LOGO_PNG, const.ICON_WIDTH, const.ICON_HEIGHT, True)
    elif "LLVM" in line:
        gpu_image = fetchImageFromUrl(const.LLVM_LOGO_SVG, const.ICON_WIDTH, const.ICON_HEIGHT, True)
    elif "Mesa" in line or "Clover" in line:
        gpu_image = fetchImageFromUrl(const.MESA_LOGO_PNG,100,const.ICON_HEIGHT, False)
    elif "Portable Computing Language" in line:
        gpu_image = fetchImageFromUrl(const.POCL_LOGO_PNG,100,const.ICON_HEIGHT, False)
    else:
        gpu_image = fetchImageFromUrl(const.TRANSPARENT_PIXBUF,100,const.ICON_HEIGHT, False)
    return gpu_image

def getLogo(line):
    if "Ubuntu" in line and "Gnome" not in line:
        logo_pixbuf  = fetchImageFromUrl(const.Ubuntu_logo,32,28, True)
    elif "Nobara" in line:
        logo_pixbuf  = fetchImageFromUrl(const.Nobara_OS_logo,32,28, True)
    elif "AnduinOS" in line:
        logo_pixbuf  = fetchImageFromUrl(const.Anduinos_logo,32,28, True)
    elif "NixOS" in line:
        logo_pixbuf  = fetchImageFromUrl(const.NixOS_logo,32,28, True)
    elif "Fedora" in line:
        logo_pixbuf = fetchImageFromUrl(const.fedora_logo,32,28, True)
    elif "COSMIC" in line:
        logo_pixbuf  = fetchImageFromUrl(const.cosmic_logo,32,28, True)
    elif "Budgie" in line and "Ubuntu" in line:
        logo_pixbuf = fetchImageFromUrl(const.Ubuntu_Budgie_logo,32,28, True)
    elif "Studio" in line and "Ubuntu" in line:
        logo_pixbuf = fetchImageFromUrl(const.Ubuntu_Studio_logo,32,28, True)
    elif "Flatpak" in line:
        logo_pixbuf = fetchImageFromUrl(const.Flatpak_logo,32,28,True)
    elif "RebornOS" in line:
        logo_pixbuf = fetchImageFromUrl(const.RebornOS_logo,32,28,True)
    elif "Kubuntu" in line:
        logo_pixbuf = fetchImageFromUrl(const.Kubuntu_logo,32,28,True)
    elif "Lubuntu" in line:
        logo_pixbuf = fetchImageFromUrl(const.Lubuntu_logo,32,28,True)
    elif "Solus" in line:       
        logo_pixbuf = fetchImageFromUrl(const.Solus_logo,32,28,True)
    elif "Mandriva" in line:       
        logo_pixbuf = fetchImageFromUrl(const.OpenMandriva_logo,32,28,True)
    elif "CachyOS" in line:       
        logo_pixbuf = fetchImageFromUrl(const.CACHYOS_LOGO_PNG,32,28,True)
    elif "Xubuntu" in line:
        logo_pixbuf = fetchImageFromUrl(const.Xubuntu_logo,32,28,True)
    elif "Arch" in line:
        logo_pixbuf = fetchImageFromUrl(const.Arch_logo,32,28, True)
    elif "Elementary" in line:
        logo_pixbuf = fetchImageFromUrl(const.Elementary_logo,32,28,True)
    elif "Debian" in line:
        logo_pixbuf = fetchImageFromUrl(const.Debian_logo,32,28,True)
    elif "opensuse" in line:
        logo_pixbuf = fetchImageFromUrl(const.Open_Suse_logo,32,28,True)
    elif "Pop" in line:
        logo_pixbuf = fetchImageFromUrl(const.Pop_os_logo,32,28,True)
    elif "MX" in line:
        logo_pixbuf = fetchImageFromUrl(const.MX_linux_logo,32,28,True)
    elif "Zorin" in line:
        logo_pixbuf = fetchImageFromUrl(const.Zorin_os_logo,32,28,True)
    elif "Mint" in line:
        logo_pixbuf = fetchImageFromUrl(const.Mint_logo,32,28, True)
    elif "Radeon" in line and "Ryzen" not in line:
        logo_pixbuf = fetchImageFromUrl(const.AMD_LOGO_PNG,32,28, False)
    elif "Ryzen" in line:
        logo_pixbuf = fetchImageFromUrl(const.AMDRYZEN_LOGO_PNG,32,28, True)
    elif "NVIDIA" in line and "Ryzen" not in line:
        logo_pixbuf = fetchImageFromUrl(const.Nvidia_logo,32,28, True)   
    elif ("Mesa" in line or "radv" in line or "llvmpipe" in line or "dozen" in line or "venus" in line or "nvk" in line or "NVK" in line) and ("LLVM" not in line):
        logo_pixbuf = fetchImageFromUrl(const.Mesa_logo,32,28, True)
    elif "LLVM" in line :
        logo_pixbuf = fetchImageFromUrl(const.LLVM_logo,32,28, True)   
    elif "AMD" in line:
        logo_pixbuf = fetchImageFromUrl(const.AMD_logo,32,28, True)      
    elif "Intel" in line:
        logo_pixbuf = fetchImageFromUrl(const.Intel_logo,32,28, True)
    elif "Manjaro" in line:
        logo_pixbuf = fetchImageFromUrl(const.Manjaro_logo,32,28, True)
    elif "sway" in line or "Sway" in line:
        logo_pixbuf = fetchImageFromUrl(const.Sway_logo,32,28, True)
    elif "Budgie" in line:
        logo_pixbuf = fetchImageFromUrl(const.Budgie_logo,32,28, True)
    elif "Unity" in line:
        logo_pixbuf = fetchImageFromUrl(const.Unity_logo,64,64, True)
    elif "GNOME" in line:
        logo_pixbuf = fetchImageFromUrl(const.Gnome_logo,32,28, True)
    elif "Fluxbox" in line:
        logo_pixbuf = fetchImageFromUrl(const.Fluxbox_logo,32,28, True)
    elif "XFCE" in line:
        logo_pixbuf = fetchImageFromUrl(const.XFCE_logo,32,28, True)
    elif "MATE" in line:
        logo_pixbuf = fetchImageFromUrl(const.Mate_logo,32,28, True)
    elif "Cinnamon" in line:
        logo_pixbuf = fetchImageFromUrl(const.Cinnamon_logo,32,28, True)
    elif "wayland" in line:
        logo_pixbuf = fetchImageFromUrl(const.Wayland_logo,32,28, True)
    elif "x11" in line:
        logo_pixbuf = fetchImageFromUrl(const.X11_logo,32,28, True)
    elif "KDE" in line:
        logo_pixbuf = fetchImageFromUrl(const.Kde_logo,32,28, True)
    elif "Rhino" in line:
        logo_pixbuf = fetchImageFromUrl(const.Rhino_Linux_logo,32,28, True)
    elif "Steam" in line:
        logo_pixbuf = fetchImageFromUrl(const.Steam_OS_logo,32,28, True)
    else:
        logo_pixbuf = fetchImageFromUrl(const.TRANSPARENT_PIXBUF,32,28,True)
    return logo_pixbuf

class ExpandDataObject(GObject.GObject):
    def __init__(self, txt: str, txt2: str):
        super(ExpandDataObject, self).__init__()
        self.data = txt
        self.data2 = txt2
        self.children = []

def add_tree_node(item):
    if not (item):
            print("no item")
            return model
    else:        
        if type(item) == Gtk.TreeListRow:
            item = item.get_item()

            print("converteu")
            print(item)  
            
        if not item.children:
            return None
        store = Gio.ListStore.new(ExpandDataObject)
        for child in item.children:
            store.append(child)
        return store

def setup_expander(widget, item):
    """Setup the widget to show in the Gtk.Listview"""
    label = Gtk.Label()
    expander = Gtk.TreeExpander.new()
 #   expander.props.indent_for_icon = True
 #   expander.props.indent_for_depth = True
    label.set_ellipsize(Pango.EllipsizeMode.END)
    expander.set_child(label)
    item.set_child(expander)

def setup(widget, item):
    """Setup the widget to show in the Gtk.Listview"""
    label = Gtk.Label()
    label.props.xalign = 0.0
    label.set_ellipsize(Pango.EllipsizeMode.END)
    item.set_child(label)

def bind_expander(widget, item):
    """bind data from the store object to the widget"""
    expander = item.get_child()
    label = expander.get_child()
    row = item.get_item()
    expander.set_list_row(row)
    obj = row.get_item()
    label.set_label(obj.data)
    label.add_css_class(css_class='parent')

def bind1(widget, item):
    """bind data from the store object to the widget"""
    label = item.get_child()
    row = item.get_item()
    obj = row.get_item()
    if "true" in obj.data2 or "Yes" in obj.data2: 
        label.remove_css_class(css_class='nothing')
        label.remove_css_class(css_class='error')
        label.add_css_class(css_class='success')
        label.set_label(obj.data2)
    elif "false" in obj.data2 or "No" in obj.data2 and "None" not in obj.data2:
        label.remove_css_class(css_class='nothing')
        label.remove_css_class(css_class='success')
        label.add_css_class(css_class='error')
        label.set_label(obj.data2)
    else:
        label.remove_css_class(css_class='error')
        label.remove_css_class(css_class='success')
        label.add_css_class(css_class='nothing')
        label.set_label(obj.data2)


class ExpandDataObject2(GObject.GObject):
    def __init__(self, txt: str, txt2: str,txt3: str,txt4: str,txt5: str):
        super(ExpandDataObject2, self).__init__()
        self.data = txt
        self.data2 = txt2
        self.data3= txt3
        self.data4= txt4
        self.data5= txt5
        self.children = []



def add_tree_node2(item):
    if not (item):
            print("no item")
            return model
    else:        
        if type(item) == Gtk.TreeListRow:
            item = item.get_item()

            print("converteu")
            print(item)  
            
        if not item.children:
            return None
        store = Gio.ListStore.new(ExpandDataObject2)
        for child in item.children:
            store.append(child)
        return store

def bind2(widget, item):
    """bind data from the store object to the widget"""
    label = item.get_child()
    row = item.get_item()
    obj = row.get_item()
    if "true" in obj.data3: 
        label.add_css_class(css_class='success')
    elif "false" in obj.data3:
        label.add_css_class(css_class='error')
    else:
        label.add_css_class(css_class='nothing')
    label.set_label(obj.data3)


def bind3(widget, item):
    """bind data from the store object to the widget"""
    label = item.get_child()
    row = item.get_item()
    obj = row.get_item()
    if "true" in obj.data4: 
        label.add_css_class(css_class='success')
    elif "false" in obj.data4:
        label.add_css_class(css_class='error')
    else:
        label.add_css_class(css_class='nothing')
    label.set_label(obj.data4)


def bind4(widget, item):
    """bind data from the store object to the widget"""
    label = item.get_child()
    row = item.get_item()
    obj = row.get_item()
    if "true" in obj.data5: 
        label.add_css_class(css_class='success')
    elif "false" in obj.data5:
        label.add_css_class(css_class='error')
    else:
        label.add_css_class(css_class='nothing')
    label.set_label(obj.data5)
